@@ -1516,6 +1516,7 @@ int CSAntTask_ResLink(CSAntTask *task)
 	char *output;
 	char *language;
 	char *version;
+	char *metadataVersion;
 	char *temp;
 	CSAntFileSet *resources;
 	const char *compiler;
@@ -1541,6 +1542,7 @@ int CSAntTask_ResLink(CSAntTask *task)
 		language = "en";
 	}
 	version = (char *)CSAntTaskParam(task, "version");
+	metadataVersion = (char *)CSAntTaskParam(task, "metadataVersion");
 	resources = CSAntFileSetLoad(task, "resources", CSAntBaseBuildDir);
 	temp = (char *)CSAntTaskParam(task, "install");
 	install = (temp && !ILStrICmp(temp, "true"));
@@ -1604,6 +1606,11 @@ int CSAntTask_ResLink(CSAntTask *task)
 			{
 				AddValueArg(&argv, &argc, "-fassembly-version=", version);
 			}
+			if(metadataVersion && metadataVersion[0] != '\0')
+			{
+				AddValueArg(&argv, &argc, "-fmetadata-version=",
+							metadataVersion);
+			}
 			AddArg(&argv, &argc, "-o");
 			AddArg(&argv, &argc, output);
 			numFiles = CSAntFileSetSize(resources);
@@ -1612,6 +1619,7 @@ int CSAntTask_ResLink(CSAntTask *task)
 				AddValueArg(&argv, &argc, "-fresources=",
 							CSAntFileSetFile(resources, file));
 			}
+			AddValueArg(&argv, &argc, "-fculture=", language);
 		}
 		else
 		{
