@@ -69,11 +69,50 @@ void CLexLineDirective(const char *text)
 	CCPreProcessorStream.lineNumber = linenum;
 }
 
+void CLexUsingDirective(const char *text)
+{
+	int endquote, len;
+	char *filename;
+	while(*text != '\0' && *text != '"' && *text != '<')
+	{
+		++text;
+	}
+	if(*text == '"')
+	{
+		endquote = '"';
+	}
+	else if(*text == '<')
+	{
+		endquote = '>';
+	}
+	else
+	{
+		return;
+	}
+	++text;
+	len = 0;
+	while(text[len] != '\0' && text[len] != endquote)
+	{
+		++len;
+	}
+	filename = ILDupNString(text, len);
+	if(!filename)
+	{
+		CCOutOfMemory();
+	}
+	if(!CCLoadLibrary(filename))
+	{
+		CCError(_("could not load the assembly `%s'"), filename);
+	}
+	ILFree(filename);
+}
+
 static void InferIntType(const char *text, CLexIntConst *value)
 {
 	int numl, numu;
 
 	/* Infer a default type for the value */
+#if 0
 	if(value->value <= (ILUInt64)127)
 	{
 		value->type = ILMachineType_Int8;
@@ -90,7 +129,8 @@ static void InferIntType(const char *text, CLexIntConst *value)
 	{
 		value->type = ILMachineType_UInt16;
 	}
-	else if(value->value <= (ILUInt64)(ILInt64)IL_MAX_INT32)
+#endif
+	if(value->value <= (ILUInt64)(ILInt64)IL_MAX_INT32)
 	{
 		value->type = ILMachineType_Int32;
 	}
@@ -130,7 +170,7 @@ static void InferIntType(const char *text, CLexIntConst *value)
 		else if(value->type == ILMachineType_Int64)
 			value->type = ILMachineType_UInt64;
 	}
-	if(numl >= 2 || (CTypeLongSize == 8 && numl > 0))
+	if(numl >= 2)
 	{
 		/* Convert the type into its "long long" version */
 		if(value->type == ILMachineType_Int8 ||
@@ -150,18 +190,18 @@ static void InferIntType(const char *text, CLexIntConst *value)
 	}
 	else if(numl > 0)
 	{
-		/* Convert the type into its "32-bit long" version */
+		/* Convert the type into its "long" version */
 		if(value->type == ILMachineType_Int8 ||
 		   value->type == ILMachineType_Int16 ||
 		   value->type == ILMachineType_Int32)
 		{
-			value->type = ILMachineType_Int32;
+			value->type = ILMachineType_NativeInt;
 		}
 		else if(value->type == ILMachineType_UInt8 ||
 		        value->type == ILMachineType_UInt16 ||
 		        value->type == ILMachineType_UInt32)
 		{
-			value->type = ILMachineType_UInt32;
+			value->type = ILMachineType_NativeUInt;
 		}
 	}
 }
@@ -494,7 +534,7 @@ void CLexParseString(char *text, ILIntString *value)
 	unsigned ch2;
 
 	/* Initialize the value to be returned */
-	if(*text == 'L' || *text == 'l')
+	if(*text == 'L' || *text == 'l' || *text == 's' || *text == 'S')
 	{
 		++text;
 	}
