@@ -1,7 +1,7 @@
 /*
  * dump_type.c - Dump types in assembly format.
  *
- * Copyright (C) 2001  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2001-2004  Southern Storm Software, Pty Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,11 +19,31 @@
  */
 
 #include "il_dumpasm.h"
+#include "il_system.h"
 
 #ifdef	__cplusplus
 extern	"C" {
 #endif
 
+/*
+ * Determine if a class has a specific name.
+ */
+static int IsSpecialClass(ILClass *classInfo, const char *name,
+						  const char *namespace)
+{
+	const char *nspace;
+	if(strcmp(ILClass_Name(classInfo), name) != 0)
+	{
+		return 0;
+	}
+	nspace = ILClass_Namespace(classInfo);
+	if(!nspace || strcmp(nspace, namespace) != 0)
+	{
+		return 0;
+	}
+	return 1;
+}
+
 void ILDumpType(FILE *stream, ILImage *image, ILType *type, int flags)
 {
 	ILType *elem;
@@ -40,73 +60,157 @@ void ILDumpType(FILE *stream, ILImage *image, ILType *type, int flags)
 
 			case IL_META_ELEMTYPE_BOOLEAN:
 			{
-				fputs("bool", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("bool", stream);
+				}
+				else
+				{
+					fputs("_Bool", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_CHAR:
 			{
-				fputs("char", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("char", stream);
+				}
+				else
+				{
+					fputs("__wchar__", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_I1:
 			{
-				fputs("int8", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("int8", stream);
+				}
+				else
+				{
+					fputs("char", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_U1:
 			{
-				fputs("unsigned int8", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("unsigned int8", stream);
+				}
+				else
+				{
+					fputs("unsigned char", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_I2:
 			{
-				fputs("int16", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("int16", stream);
+				}
+				else
+				{
+					fputs("short", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_U2:
 			{
-				fputs("unsigned int16", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("unsigned int16", stream);
+				}
+				else
+				{
+					fputs("unsigned short", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_I4:
 			{
-				fputs("int32", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("int32", stream);
+				}
+				else
+				{
+					fputs("int", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_U4:
 			{
-				fputs("unsigned int32", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("unsigned int32", stream);
+				}
+				else
+				{
+					fputs("unsigned int", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_I8:
 			{
-				fputs("int64", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("int64", stream);
+				}
+				else
+				{
+					fputs("long long", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_U8:
 			{
-				fputs("unsigned int64", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("unsigned int64", stream);
+				}
+				else
+				{
+					fputs("unsigned long long", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_R4:
 			{
-				fputs("float32", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("float32", stream);
+				}
+				else
+				{
+					fputs("float", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_R8:
 			{
-				fputs("float64", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("float64", stream);
+				}
+				else
+				{
+					fputs("double", stream);
+				}
 			}
 			break;
 
@@ -118,19 +222,40 @@ void ILDumpType(FILE *stream, ILImage *image, ILType *type, int flags)
 
 			case IL_META_ELEMTYPE_I:
 			{
-				fputs("native int", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("native int", stream);
+				}
+				else
+				{
+					fputs("__native__ int", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_U:
 			{
-				fputs("native unsigned int", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("native unsigned int", stream);
+				}
+				else
+				{
+					fputs("__native__ unsigned int", stream);
+				}
 			}
 			break;
 
 			case IL_META_ELEMTYPE_R:
 			{
-				fputs("native float", stream);
+				if((flags & IL_DUMP_C_TYPES) == 0)
+				{
+					fputs("native float", stream);
+				}
+				else
+				{
+					fputs("long double", stream);
+				}
 			}
 			break;
 
@@ -154,7 +279,10 @@ void ILDumpType(FILE *stream, ILImage *image, ILType *type, int flags)
 	}
 	else if(ILType_IsValueType(type))
 	{
-		fputs("valuetype ", stream);
+		if((flags & IL_DUMP_C_TYPES) == 0)
+		{
+			fputs("valuetype ", stream);
+		}
 		ILDumpClassName(stream, image, ILType_ToValueType(type), flags);
 	}
 	else if(type && ILType_IsComplex(type))
@@ -246,20 +374,61 @@ void ILDumpType(FILE *stream, ILImage *image, ILType *type, int flags)
 			case IL_TYPE_COMPLEX_CMOD_REQD:
 			{
 				ILDumpType(stream, image, type->un.modifier__.type__, flags);
-				fputs(" modreq(", stream);
-				ILDumpClassName(stream, image,
-								type->un.modifier__.info__, flags);
-				putc(')', stream);
+				if(IsSpecialClass(type->un.modifier__.info__,
+								  "IsVolatile",
+								  "System.Runtime.CompilerServices") &&
+				   (flags & IL_DUMP_C_TYPES) != 0)
+				{
+					fputs(" volatile", stream);
+				}
+				else
+				{
+					fputs(" modreq(", stream);
+					ILDumpClassName(stream, image,
+									type->un.modifier__.info__, flags);
+					putc(')', stream);
+				}
 			}
 			break;
 
 			case IL_TYPE_COMPLEX_CMOD_OPT:
 			{
-				ILDumpType(stream, image, type->un.modifier__.type__, flags);
-				fputs(" modopt(", stream);
-				ILDumpClassName(stream, image,
-								type->un.modifier__.info__, flags);
-				putc(')', stream);
+				if(IsSpecialClass(type->un.modifier__.info__,
+								  "IsConst", "OpenSystem.C") &&
+				   (flags & IL_DUMP_C_TYPES) != 0)
+				{
+					if(ILType_IsPrimitive(type->un.modifier__.type__))
+					{
+						/* Put the "const" before primitive types because
+						   it is the more natural position for types like
+						   "const char *" */
+						fputs("const ", stream);
+						ILDumpType(stream, image, type->un.modifier__.type__,
+								   flags);
+					}
+					else
+					{
+						ILDumpType(stream, image, type->un.modifier__.type__,
+								   flags);
+						fputs(" const", stream);
+					}
+				}
+				else if(IsSpecialClass(type->un.modifier__.info__,
+								  	   "IsFunctionPointer", "OpenSystem.C") &&
+				        (flags & IL_DUMP_C_TYPES) != 0)
+				{
+					ILDumpType(stream, image, type->un.modifier__.type__,
+							   flags);
+				}
+				else
+				{
+					ILDumpType(stream, image, type->un.modifier__.type__,
+							   flags);
+					fputs(" modopt(", stream);
+					ILDumpClassName(stream, image,
+									type->un.modifier__.info__, flags);
+					putc(')', stream);
+				}
 			}
 			break;
 
@@ -321,7 +490,10 @@ void ILDumpType(FILE *stream, ILImage *image, ILType *type, int flags)
 			{
 				if((type->kind__ & IL_TYPE_COMPLEX_METHOD) != 0)
 				{
-					fputs("method ", stream);
+					if((flags & IL_DUMP_C_TYPES) == 0)
+					{
+						fputs("method ", stream);
+					}
 					ILDumpMethodType(stream, image, type, flags, 0, 0, 0);
 				}
 				else
@@ -517,11 +689,15 @@ static void DumpMethodType(FILE *stream, ILImage *image, ILType *type,
 	flags &= ~IL_DUMP_GENERIC_PARAMS;
 
 	/* Dump the calling conventions for the method */
-	callingConventions = ILType_CallConv(type);
-	ILDumpFlags(stream, callingConventions, ILMethodCallConvFlags, 0);
+	if((flags & IL_DUMP_C_TYPES) == 0)
+	{
+		callingConventions = ILType_CallConv(type);
+		ILDumpFlags(stream, callingConventions, ILMethodCallConvFlags, 0);
+	}
 
 	/* Dump the return type */
-	DumpParamType(stream, image, methodInfo, ILTypeGetReturn(type), 0, flags);
+	DumpParamType(stream, image, methodInfo,
+				  ILTypeGetReturnWithPrefixes(type), 0, flags);
 	putc(' ', stream);
 
 	/* Dump the class name and method name */
@@ -545,6 +721,10 @@ static void DumpMethodType(FILE *stream, ILImage *image, ILType *type,
 			ILDumpIdentifier(stream, methodName, 0, flags);
 		}
 	}
+	else if((flags & IL_DUMP_C_TYPES) != 0)
+	{
+		fputs("(*)", stream);
+	}
 	else
 	{
 		putc('*', stream);
