@@ -117,7 +117,10 @@ ILCallFrame *_ILAllocCallFrame(ILExecThread *thread)
 			ILThreadAtomicStart(); \
 			thread->managedSafePointFlags &= ~_IL_MANAGED_SAFEPOINT_THREAD_SUSPEND; \
 			ILThreadAtomicEnd(); \
-			_ILExecThreadSuspendThread(thread, thread->supportThread); \
+			if (ILThreadGetState(thread->supportThread) & IL_TS_SUSPEND_REQUESTED) \
+			{ \
+				_ILExecThreadSuspendThread(thread, thread->supportThread); \
+			} \
 		} \
 	}
 
@@ -214,14 +217,13 @@ ILUInt32 _ILStackWordsForType(ILExecThread *thread, ILType *type)
 	}
 }
 
-#ifdef IL_CONFIG_VARARGS
-
 /*
- * Pack a set of arguments into a vararg "Object[]" array.
+ * Pack a set of arguments into a params "Object[]" array.
  * Returns the number of stack words to pop from the function,
- * and the new array in "*array".
+ * and the new array in "*array". This is not included in VarArgs
+ * as this is needed by non-vararg operations like BeginInvoke
  */
-ILUInt32 _ILPackVarArgs(ILExecThread *thread, CVMWord *stacktop,
+ILUInt32 _ILPackCVMStackArgs(ILExecThread *thread, CVMWord *stacktop,
 							ILUInt32 firstParam, ILUInt32 numArgs,
 							ILType *callSiteSig, void **array)
 {
@@ -512,8 +514,6 @@ ILUInt32 _ILPackVarArgs(ILExecThread *thread, CVMWord *stacktop,
 	return height;
 }
 
-#endif /* IL_CONFIG_VARARGS */
-
 /*
  * Get the number of parameter words for a method.
  */
@@ -1970,8 +1970,9 @@ VMCASE(COP_PREFIX_PACK_VARARGS):
 #ifdef IL_CONFIG_VARARGS	
 
 	COPY_STATE_TO_THREAD();
-	tempNum = _ILPackVarArgs(thread, stacktop, CVMP_ARG_WORD, CVMP_ARG_WORD2,
-						  CVMP_ARG_WORD2_PTR(ILType *), &tempptr);
+	tempNum = _ILPackCVMStackArgs
+		(thread, stacktop, CVMP_ARG_WORD, CVMP_ARG_WORD2,
+		 CVMP_ARG_WORD2_PTR(ILType *), &tempptr);
 	RESTORE_STATE_FROM_THREAD();
 
 	stacktop -= tempNum;
