@@ -380,6 +380,52 @@ ILNativeInt _IL_AssemblyBuilder_ClrGetItemFromToken(ILExecThread *_thread,
 	}
 }
 
+/*
+ * private static IntPtr ClrAttributeCreate(IntPtr assembly, IntPtr ctor,
+ *											byte[] blob);
+ */
+ILNativeInt _IL_AssemblyBuilder_ClrAttributeCreate
+		(ILExecThread * _thread, ILNativeInt assembly,
+		 ILNativeInt ctor, System_Array * blob)
+{
+	if(assembly && blob)
+	{
+		ILMember * member = ILProgramItemToMember((ILProgramItem*)ctor);
+		ILAttribute *attr = ILAttributeCreate
+			(ILProgramItem_Image((ILAssembly *)assembly), 0);
+		member = ILMemberImport(ILProgramItem_Image((ILAssembly *) assembly), member);
+		if(!attr)
+		{
+			return 0;
+		}
+		ILAttributeSetType(attr, ILToProgramItem(member));
+		if(!ILAttributeSetValue(attr, ArrayToBuffer(blob),
+								(unsigned long)(long)(blob->length)))
+		{
+			return 0;
+		}
+		return (ILNativeInt)attr;
+	}
+	else
+	{
+		return 0;
+	}
+}
+
+/*
+ * private static void ClrAttributeAddToItem(IntPtr item, IntPtr attribute);
+ */
+void _IL_AssemblyBuilder_ClrAttributeAddToItem
+		(ILExecThread * _thread, ILNativeInt item, ILNativeInt attribute)
+{
+	if(item && attribute)
+	{
+		ILProgramItemAddAttribute
+			((ILProgramItem *)item, (ILAttribute *)attribute);
+		ILProgramItemConvertAttrs((ILProgramItem *)item);
+	}
+}
+
 /*
  * private static IntPtr ClrEventCreate(IntPtr classInfo, String name,
  *                                      IntPtr type, EventAttributes attrs);
@@ -1952,11 +1998,19 @@ ILNativeInt _IL_ParameterBuilder_ClrParameterCreate(ILExecThread *_thread,
 	IL_METADATA_WRLOCK(_thread);
 
 	methodInfo = (ILMethod *)method;
-	if (!(str = (const char *)ILStringToAnsi(_thread, name)))
+	if(name != NULL)
 	{
-		IL_METADATA_UNLOCK(_thread);
-		ILExecThreadThrowOutOfMemory(_thread);
-		return 0;
+		if (!(str = (const char *)ILStringToAnsi(_thread, name)))
+		{
+			IL_METADATA_UNLOCK(_thread);
+			ILExecThreadThrowOutOfMemory(_thread);
+			return 0;
+		}
+	}
+	else
+	{
+		// Parameter names can be NULL
+		str = NULL;
 	}
 	if (!(retval = ILParameterCreate(methodInfo, 0, str, (ILUInt32)attributes, (ILUInt32)position)))
 	{
