@@ -897,7 +897,7 @@ System_Array *_IL_Assembly_GetExportedTypes(ILExecThread *_thread,
 	if(item && _ILClrCheckItemAccess(_thread, item)) 
 	{
 		num = ILImageNumTokens (image, IL_META_TOKEN_TYPE_DEF);
-		array = (System_Array *)ILExecThreadNew(_thread, "[oSystem.Object;",
+		array = (System_Array *)ILExecThreadNew(_thread, "[oSystem.Type;",
 												"(Ti)V", (ILVaInt)num);
 		if(!array)
 		{
@@ -921,7 +921,7 @@ System_Array *_IL_Assembly_GetExportedTypes(ILExecThread *_thread,
 	}
 	/* Invalid item, or insufficient access: return a zero-element array */
 	return (System_Array *)ILExecThreadNew
-				(_thread, "[oSystem.Object;", "(Ti)V", (ILVaInt)0);
+				(_thread, "[oSystem.Type;", "(Ti)V", (ILVaInt)0);
 }
 
 /*
@@ -1353,7 +1353,7 @@ System_Array *_IL_Assembly_GetTypes(ILExecThread *_thread, ILObject *_this)
 	if(item && _ILClrCheckItemAccess(_thread, item)) 
 	{
 		num = ILImageNumTokens (image, IL_META_TOKEN_TYPE_DEF);
-		array = (System_Array *)ILExecThreadNew(_thread, "[oSystem.Object;",
+		array = (System_Array *)ILExecThreadNew(_thread, "[oSystem.Type;",
 												"(Ti)V", (ILVaInt)num);
 		if(!array)
 		{
@@ -1377,7 +1377,7 @@ System_Array *_IL_Assembly_GetTypes(ILExecThread *_thread, ILObject *_this)
 	}
 	/* Invalid item, or insufficient access: return a zero-element array */
 	return (System_Array *)ILExecThreadNew
-				(_thread, "[oSystem.Object;", "(Ti)V", (ILVaInt)0);
+				(_thread, "[oSystem.Type;", "(Ti)V", (ILVaInt)0);
 }
 
 /*
@@ -1474,6 +1474,16 @@ ILObject *_IL_Assembly_LoadFromFile(ILExecThread *thread,
 
 	/* TODO: check security permissions */
 
+	/* Load from context if it exists already */
+
+	image = ILContextGetFile(thread->process->context, filename);
+
+	if(image != NULL)
+	{
+		*error = LoadError_OK;
+		return ImageToAssembly(thread, image);
+	}
+
 	/* Attempt to load the file */
 	loadError = ILImageLoadFromFile(filename, thread->process->context,
 									&image, IL_LOADFLAG_FORCE_32BIT, 0);
@@ -2450,6 +2460,7 @@ void _IL_ClrField_SetValueInternal(ILExecThread *thread, ILObject *_this,
 {
 	ILField *field;
 	ILType *type;
+	ILType *objectType;
 	void *ptr;
 
 	/* Get the program item for the field reference */
@@ -2468,9 +2479,11 @@ void _IL_ClrField_SetValueInternal(ILExecThread *thread, ILObject *_this,
 	}
 
 	/* Is the field literla, static or instance? */
-	if(ILField_IsLiteral(field))
+	if(ILField_IsLiteral(field) || ILField_IsInitOnly(field))
 	{
 		/* Cannot set literal fields */
+		ILExecThreadThrowSystem
+			(thread, "System.FieldAccessException", 0);
 		return;
 	}
 	else if(ILField_IsStatic(field))
@@ -2502,6 +2515,36 @@ void _IL_ClrField_SetValueInternal(ILExecThread *thread, ILObject *_this,
 		ptr = (void *)(((unsigned char *)obj) + field->offset);
 	}
 
+	/* Handle null value (null has no type) */
+	if (value == 0)
+	{
+		if (ILTypeIsReference(type))
+		{
+			/* Set null value on a reference field */
+			
+			*((ILObject **)ptr) = 0;
+			
+			return;
+		}
+		else
+		{
+			/* Null on a value type field is equivalent to setting
+			   the field to the value type default value */
+			
+			ILMemZero(ptr, ILSizeOfType(thread, type));
+			
+			return;
+		}
+	}
+		
+	objectType = ILClassToType(GetObjectClass(value));
+				
+	if(!ILTypeAssignCompatible(ILProgramItem_Image(field), objectType, type))
+	{
+		ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
+		return;
+	}
+
 	/* Fetch the value, box it, and return */
 	if(ILTypeIsReference(type))
 	{
@@ -2623,6 +2666,102 @@ ILNativeInt _IL_RuntimeMethodHandle_GetFunctionPointer
 	}
 }
 
+#define UNBOX_AND_ASSIGN(result, value) \
+					if(ILExecThreadUnbox(thread, \
+										 objectType, \
+										 paramObject, \
+										 (void*)&(value)))\
+					{\
+						(result) = (ILNativeFloat)(value);\
+					}\
+					else \
+					{ \
+						return NULL;\
+					}
+					
+
+/* If promotions are possible to floating point , do it (even at the loss of
+ * significant digits).
+ */
+static ILObject* PromoteToFloat(ILExecThread *thread, ILType *paramType,
+								ILType *objectType, ILObject* paramObject)
+{
+	ILNativeFloat nativeValue;
+	ILInt32 intValue;
+	ILUInt32 uintValue;
+	ILInt64 longValue;
+	ILUInt64 ulongValue;
+	ILFloat floatValue;
+	switch(ILType_ToElement(paramType))
+	{
+		case IL_META_ELEMTYPE_R8: // float -> double coercion
+		{
+			if(ILType_ToElement(objectType) == IL_META_ELEMTYPE_R4)
+			{
+				if(ILExecThreadUnboxFloat(thread, 
+									 objectType, 
+									 paramObject, 
+									 (void*)&(floatValue)))
+				{
+					(nativeValue) = (ILNativeFloat)(floatValue);
+				}
+				else 
+				{ 
+					return NULL;
+				}
+			}
+		}
+		/* fall through */
+		case IL_META_ELEMTYPE_R4:
+		{
+			switch(ILType_ToElement(objectType))
+			{
+				case IL_META_ELEMTYPE_I1:
+				case IL_META_ELEMTYPE_I2:
+				case IL_META_ELEMTYPE_I4:
+				{
+					UNBOX_AND_ASSIGN(nativeValue, intValue);
+				}
+				break;
+
+				case IL_META_ELEMTYPE_U1:
+				case IL_META_ELEMTYPE_U2:
+				case IL_META_ELEMTYPE_U4:
+				{
+					UNBOX_AND_ASSIGN(nativeValue, uintValue);
+				}
+				break;
+				
+				case IL_META_ELEMTYPE_I8:
+				{
+					UNBOX_AND_ASSIGN(nativeValue, longValue);
+				}
+				break;
+
+				case IL_META_ELEMTYPE_U8:
+				{
+					UNBOX_AND_ASSIGN(nativeValue, ulongValue);
+				}
+				break;
+
+				default:
+				{
+					return NULL;
+				}
+				break;
+			}
+		}
+		break;
+		default:
+		{
+			return NULL;
+		}
+		break;
+	}
+
+	return ILExecThreadBoxFloat(thread, paramType, &nativeValue);
+}
+
 /*
  * Invoke a method via reflection.
  */
@@ -2707,7 +2846,7 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 				   with byref's target type */
 
 				objectType = ILClassToType(GetObjectClass(paramObject));
-				
+
 				if(!ILTypeAssignCompatible(image, objectType, ILType_Ref(paramType)))
 				{
 					ILExecThreadThrowSystem(thread, "System.ArgumentException", 0);
@@ -2751,6 +2890,16 @@ static ILObject *InvokeMethod(ILExecThread *thread, ILMethod *method,
 
 				objectType = ILClassToType(GetObjectClass(paramObject));
 				
+				if(ILType_IsPrimitive(paramType) && ILType_IsPrimitive(objectType))
+				{
+					ILObject *promotedObject = PromoteToFloat(thread, paramType, objectType, paramObject);
+					if(promotedObject != NULL)
+					{
+						paramObject = promotedObject;
+						objectType = paramType;
+					}
+				}
+				
 				/* Make sure the type passed matches the param type */
 				if(!ILTypeAssignCompatible(image, objectType, paramType))
 				{
@@ -3261,7 +3410,7 @@ System_Array *_IL_Module_GetTypes(ILExecThread *_thread, ILObject *_this)
 	{
 		ILImage *image = ILProgramItem_Image(module);
 		num = ILImageNumTokens (image, IL_META_TOKEN_TYPE_DEF);
-		array = (System_Array *)ILExecThreadNew(_thread, "[oSystem.Object;",
+		array = (System_Array *)ILExecThreadNew(_thread, "[oSystem.Type;",
 												"(Ti)V", (ILVaInt)num);
 		if(!array)
 		{
