@@ -210,17 +210,22 @@ ILObject *_IL_Interlocked_CompareExchange_RObjectObjectObject
  */
 static void __PrivateThreadStart(void *objectArg)
 {
-	ILMethod *method;
 	ILExecValue result;
 	ILExecThread *thread;
 
 	thread = ILExecThreadCurrent();
 
-	/* Get the ThreadStart.Invoke method */
-	method = ILExecThreadLookupMethod(thread, "System.Threading.ThreadStart", "Invoke", "(T)V");
-
-	/* Invoke the ThreadStart delegate */
-	ILExecThreadCall(thread, method, &result, ((System_Thread *)thread->clrThread)->start);
+	_ILCallMethod
+		(
+			thread,
+			((System_Thread *)thread->clrThread)->start->methodInfo,
+			_ILCallUnpackDirectResult,
+			&result,
+			0,
+			((System_Thread *)thread->clrThread)->start->target,
+			_ILCallPackVParams,
+			0
+		);
 
 	/* Print out any uncaught exceptions */
 	if (ILExecThreadHasException(thread)
@@ -329,7 +334,7 @@ void _IL_Thread_Suspend(ILExecThread *thread, ILObject *_this)
  */
 void _IL_Thread_Resume(ILExecThread *thread, ILObject *_this)
 {	
-	ILThreadResume(((System_Thread *)_this)->privateData);
+	_ILExecThreadResumeThread(thread, ((System_Thread *)_this)->privateData);
 }
 
 /*
@@ -611,7 +616,7 @@ void _IL_Thread_InternalSetPriority(ILExecThread *thread, ILObject *_this,
  */
 ILInt32 _IL_Thread_InternalGetState(ILExecThread *thread, ILObject *_this)
 {
-	return ILThreadGetState(((System_Thread *)_this)->privateData);
+	return _ILExecThreadGetState(thread, ((System_Thread *)_this)->privateData);
 }
 
 /*
