@@ -64,22 +64,22 @@ extern	"C" {
  * -1 is the list terminator.  The floating point register numbers
  * must include the MD_FREG_MASK value.
  */
-#define	MD_FREG_0		-1
-#define	MD_FREG_1		-1
-#define	MD_FREG_2		-1
-#define	MD_FREG_3		-1
-#define	MD_FREG_4		-1
-#define	MD_FREG_5		-1
-#define	MD_FREG_6		-1
-#define	MD_FREG_7		-1
-#define	MD_FREG_8		-1
-#define	MD_FREG_9		-1
-#define	MD_FREG_10		-1
-#define	MD_FREG_11		-1
-#define	MD_FREG_12		-1
-#define	MD_FREG_13		-1
-#define	MD_FREG_14		-1
-#define	MD_FREG_15		-1
+#define	MD_FREG_0		-1 /* TODO: enable FP */
+#define	MD_FREG_1		(MD_FREG_MASK | PPC_F1)
+#define	MD_FREG_2		(MD_FREG_MASK | PPC_F2)
+#define	MD_FREG_3		(MD_FREG_MASK | PPC_F3)
+#define	MD_FREG_4		(MD_FREG_MASK | PPC_F4)
+#define	MD_FREG_5		(MD_FREG_MASK | PPC_F5)
+#define	MD_FREG_6		(MD_FREG_MASK | PPC_F6)
+#define	MD_FREG_7		(MD_FREG_MASK | PPC_F7)
+#define	MD_FREG_8		(MD_FREG_MASK | PPC_F8)
+#define	MD_FREG_9		(MD_FREG_MASK | PPC_F9)
+#define	MD_FREG_10		(MD_FREG_MASK | PPC_F10)
+#define	MD_FREG_11		(MD_FREG_MASK | PPC_F11)
+#define	MD_FREG_12		(MD_FREG_MASK | PPC_F12)
+#define	MD_FREG_13		(MD_FREG_MASK | PPC_F13)
+#define	MD_FREG_14		(MD_FREG_MASK | PPC_F14)
+#define	MD_FREG_15		(MD_FREG_MASK | PPC_F15)
 
 /*
  * Set this to a non-zero value if floating-point registers are organised
@@ -132,7 +132,7 @@ extern	"C" {
  * Set it to zero if integer division is too hard to be performed
  * inline using a simple opcode.
  */
-#define	MD_HAS_INT_DIVISION			0	/* TODO */
+#define	MD_HAS_INT_DIVISION			1
 
 /*
  * Set to 1 if 64-bit register pairs are stored in little-endian order.
@@ -177,14 +177,22 @@ typedef ppc_inst_ptr	md_inst_ptr;
  * is at a particular memory location.  If the system does not use
  * floating-point registers, then load onto the top of the stack.
  */
-#define	md_load_const_float_32(inst,reg,mem)	do { ; } while (0)
+#define	md_load_const_float_32(inst,freg,mem)	\
+			do { \
+				ppc_mov_reg_imm((inst), PPC_WORK, mem);\
+				ppc_load_membase_float_32((inst), (freg) & (~MD_FREG_MASK), PPC_WORK, 0);\
+			}while(0)
 
 /*
  * Load a 64-bit floating-point constant into a register.  The constant
  * is at a particular memory location.  If the system does not use
  * floating-point registers, then load onto the top of the stack.
  */
-#define	md_load_const_float_64(inst,reg,mem)	do { ; } while (0)
+#define	md_load_const_float_64(inst,freg,mem)   \
+			do { \
+				ppc_mov_reg_imm((inst), PPC_WORK, mem);\
+				ppc_load_membase_float_64((inst), (freg) & (~MD_FREG_MASK), PPC_WORK, 0);\
+			}while(0)
 
 /*
  * Load the 32-bit constant zero into a register.  This will zero-extend
@@ -249,15 +257,15 @@ typedef ppc_inst_ptr	md_inst_ptr;
 /*
  * Load a floating-point value from an offset from a pointer register.
  * If the system uses floating-point registers, then the value is
- * loaded into "reg".  Otherwise it is loaded onto the top of the
+ * loaded into "freg".  Otherwise it is loaded onto the top of the
  * floating-point stack.
  */
-#define	md_load_membase_float_32(inst,reg,basereg,offset)	\
-			do { ; } while (0)
-#define	md_load_membase_float_64(inst,reg,basereg,offset)	\
-			do { ; } while (0)
-#define	md_load_membase_float_native(inst,reg,basereg,offset)	\
-			do { ; } while (0)
+#define	md_load_membase_float_32(inst,freg,basereg,offset)	\
+			ppc_load_membase_float_32(inst, (freg) & (~MD_FREG_MASK), basereg, offset)
+#define	md_load_membase_float_64(inst,freg,basereg,offset)	\
+			ppc_load_membase_float_64(inst, (freg) & (~MD_FREG_MASK), basereg, offset)
+#define	md_load_membase_float_native(inst,freg,basereg,offset)	\
+			ppc_load_membase_float_64(inst, (freg) & (~MD_FREG_MASK), basereg, offset)
 
 /*
  * Store a 32-bit word register to an offset from a pointer register.
@@ -311,12 +319,12 @@ typedef ppc_inst_ptr	md_inst_ptr;
  * stored from "reg".  Otherwise it is stored from the top of the
  * floating-point stack.
  */
-#define	md_store_membase_float_32(inst,reg,basereg,offset)	\
-			do { ; } while (0)
-#define	md_store_membase_float_64(inst,reg,basereg,offset)	\
-			do { ; } while (0)
-#define	md_store_membase_float_native(inst,reg,basereg,offset)	\
-			do { ; } while (0)
+#define	md_store_membase_float_32(inst,freg,basereg,offset)	\
+			ppc_store_membase_float_32(inst, (freg) & (~MD_FREG_MASK), basereg, offset)
+#define	md_store_membase_float_64(inst,freg,basereg,offset)	\
+			ppc_store_membase_float_64(inst, (freg) & (~MD_FREG_MASK), basereg, offset)
+#define	md_store_membase_float_native(inst,freg,basereg,offset)	\
+			ppc_store_membase_float_64(inst, (freg) & (~MD_FREG_MASK), basereg, offset)
 
 /*
  * Add an immediate value to a register.
@@ -340,13 +348,21 @@ typedef ppc_inst_ptr	md_inst_ptr;
 #define	md_mul_reg_reg_word_32(inst,reg1,reg2)	\
 			ppc_alu_reg_dss((inst), PPC_MUL, (reg1), (reg1), (reg2))
 #define	md_div_reg_reg_word_32(inst,reg1,reg2)	\
-			do { ; } while (0)
+				ppc_alu_reg_dss((inst), PPC_DIV, (reg1), (reg1), (reg2))
 #define	md_udiv_reg_reg_word_32(inst,reg1,reg2)	\
-			do { ; } while (0)
+				ppc_alu_reg_dss((inst), PPC_DIV_UN , (reg1), (reg1), (reg2))
 #define	md_rem_reg_reg_word_32(inst,reg1,reg2)	\
-			do { ; } while (0)
+			do {\
+				ppc_alu_reg_dss((inst), PPC_DIV, PPC_WORK, (reg1), (reg2));\
+				ppc_alu_reg_dss((inst), PPC_MUL, PPC_WORK, PPC_WORK, (reg2));\
+				ppc_alu_reg_dss((inst), PPC_SUBF, (reg1), PPC_WORK, (reg1));\
+			} while (0)
 #define	md_urem_reg_reg_word_32(inst,reg1,reg2)	\
-			do { ; } while (0)
+			do {\
+				ppc_alu_reg_dss((inst), PPC_DIV_UN, PPC_WORK, (reg1), (reg2));\
+				ppc_alu_reg_dss((inst), PPC_MUL, PPC_WORK, PPC_WORK, (reg2));\
+				ppc_alu_reg_dss((inst), PPC_SUBF, (reg1), PPC_WORK, (reg1));\
+			} while (0)
 #define	md_neg_reg_word_32(inst,reg)	\
 			ppc_alu_reg_ds((inst), PPC_NEG, (reg), (reg))
 #define	md_and_reg_reg_word_32(inst,reg1,reg2)	\
@@ -420,16 +436,62 @@ typedef ppc_inst_ptr	md_inst_ptr;
 /*
  * Truncate floating point values to 32-bit or 64-bit.
  */
-#define	md_reg_to_float_32(inst,reg)	\
-			do { ; } while (0)
-#define	md_reg_to_float_64(inst,reg)	\
-			do { ; } while (0)
+#define	md_reg_to_float_32(inst, freg)	\
+		do { \
+			int __freg32 = (freg) & (~MD_FREG_MASK);\
+			ppc_fpu_fsrp(inst, __freg32, __freg32);\
+		}while(0)
+#define	md_reg_to_float_64(inst,freg)	\
+			do { /* it is f64 by default for all operations \
+			        All f32 operations are rounded back after \
+					operation */; } while (0)
 
 /*
  * Swap the top two items on the floating-point stack.
  */
 #define	md_freg_swap(inst)		do { ; } while (0)
 
+/*
+ * Floating point arithmetic operations
+ */
+#define	md_add_reg_reg_float(inst,reg1,reg2)	\
+		do { \
+			int __freg1 = (reg1) & (~MD_FREG_MASK);\
+			int __freg2 = (reg2) & (~MD_FREG_MASK);\
+			ppc_fpu_reg_dab(inst, 1 /* f64 */ , PPC_FADD, \
+								__freg1, __freg1, __freg2);\
+		} while(0)
+#define	md_sub_reg_reg_float(inst,reg1,reg2)	\
+		do { \
+			int __freg1 = (reg1) & (~MD_FREG_MASK);\
+			int __freg2 = (reg2) & (~MD_FREG_MASK);\
+			ppc_fpu_reg_dab(inst, 1 /* f64 */ , PPC_FSUB, \
+								__freg1, __freg1, __freg2);\
+		} while(0)
+#define	md_mul_reg_reg_float(inst,reg1,reg2)	\
+		do { \
+			int __freg1 = (reg1) & (~MD_FREG_MASK);\
+			int __freg2 = (reg2) & (~MD_FREG_MASK);\
+			ppc_fpu_reg_dac(inst, 1 /* f64 */ , PPC_FMUL, \
+								__freg1, __freg1, __freg2);\
+		} while(0)
+#define	md_div_reg_reg_float(inst,reg1,reg2)	\
+		do { \
+			int __freg1 = (reg1) & (~MD_FREG_MASK);\
+			int __freg2 = (reg2) & (~MD_FREG_MASK);\
+			ppc_fpu_reg_dab(inst, 1 /* f64 */ , PPC_FDIV, \
+								__freg1, __freg1, __freg2);\
+		} while(0)
+#define	md_rem_reg_reg_float(inst,reg1,reg2,used)	\
+		TODO_trap(inst)
+#define	md_neg_reg_float(inst,reg)	\
+		do { \
+			int __freg1 = (reg) & (~MD_FREG_MASK);\
+			ppc_fpu_reg_dab(inst, 1, PPC_FNEG, __freg1, 0, __freg1);\
+		}while(0)
+#define	md_cmp_reg_reg_float(inst,dreg,sreg1,sreg2,lessop)	\
+		TODO_trap(inst)
+
 /*
  * Jump back into the CVM interpreter to execute the instruction
  * at "pc".  If "label" is non-NULL, then it indicates the address
@@ -438,34 +500,17 @@ typedef ppc_inst_ptr	md_inst_ptr;
 #define	md_jump_to_cvm(inst,pc,label)	\
 			do { \
 				int offset; \
+				ppc_mov_reg_imm(inst, MD_REG_PC, (int)pc); \
 				if(!(label)) \
 				{ \
-					/* Jump to the contents of the specified PC */ \
-					arm_load_membase((inst), MD_REG_PC, ARM_PC, 0); \
-					arm_load_membase((inst), ARM_PC, MD_REG_PC, 0); \
-					*((inst)++) = (unsigned int)(pc); \
+					/* Jump to the contents of the specified PC */  \
+					ppc_load_membase(inst, PPC_WORK, MD_REG_PC, 0); \
 				} \
 				else \
 				{ \
-					/* Load "pc" back into the CVM interpreter's frame */ \
-					/* and then jump directly to instruction handler at */ \
-					/* "label".  This avoids the need for an indirect */ \
-					/* jump instruction */ \
-					arm_load_membase(unroll->out, MD_REG_PC, ARM_PC, 0); \
-					offset = (int)(((unsigned char *)(label)) - \
-				       			  (((unsigned char *)(inst)) + 8)); \
-					if(offset >= -0x04000000 && offset < 0x04000000) \
-					{ \
-						arm_jump_imm((inst), offset); \
-						*((inst)++) = (unsigned int)(pc); \
-					} \
-					else \
-					{ \
-						arm_load_membase(unroll->out, ARM_PC, ARM_PC, 0); \
-						*((inst)++) = (unsigned int)(pc); \
-						*((inst)++) = (unsigned int)(label); \
-					} \
+					ppc_mov_reg_imm(inst, PPC_WORK, (label));\
 				} \
+				ppc_jump_reg(inst, PPC_WORK); \
 			} while (0)
 
 /*
@@ -473,10 +518,17 @@ typedef ppc_inst_ptr	md_inst_ptr;
  */
 #define	md_switch(inst,reg,table)	\
 			do { \
-				arm_load_membase((inst), ARM_WORK, ARM_PC, 4); \
-				arm_load_memindex((inst), MD_REG_PC, ARM_WORK, (reg)); \
-				arm_load_membase((inst), ARM_PC, MD_REG_PC, 0); \
-				*((inst)++) = (unsigned int)(table); \
+					/* pc = table[reg] */\
+					ppc_mov_reg_imm(inst, PPC_WORK, (int)(table));\
+					/* cannot use ppc_load_memindex as that will clobber 
+					PPC_WORK instead of using reg as the shift dest - so
+					essentially same code but different clobbering
+					*/\
+					ppc_alu_rlwinm(inst, reg, reg, 2, 0, 29);\
+					ppc_memindex_common(inst, MD_REG_PC, PPC_WORK, reg, 0, 23);\
+					/* goto *pc; */\
+					ppc_load_membase(inst, PPC_WORK, MD_REG_PC, 0);\
+					ppc_jump_reg(inst, PPC_WORK);\
 			} while (0)
 
 /*
@@ -514,25 +566,25 @@ typedef ppc_inst_ptr	md_inst_ptr;
  * Set a register to a 0 or 1 value based on a condition.
  */
 extern md_inst_ptr _md_ppc_setcc(md_inst_ptr inst, int reg,
-								 int cond, int invcond);
+									int cond);
 #define	md_seteq_reg(inst,reg)	\
 			do { (inst) = _md_ppc_setcc \
-					((inst), (reg), PPC_CC_EQ, PPC_CC_NE); } while (0)
+					((inst), (reg), PPC_CC_EQ); } while (0)
 #define	md_setne_reg(inst,reg)	\
 			do { (inst) = _md_ppc_setcc \
-					((inst), (reg), PPC_CC_NE, PPC_CC_EQ); } while (0)
+					((inst), (reg), PPC_CC_NE); } while (0)
 #define	md_setlt_reg(inst,reg)	\
 			do { (inst) = _md_ppc_setcc \
-					((inst), (reg), PPC_CC_LT, PPC_CC_GE); } while (0)
+					((inst), (reg), PPC_CC_LT); } while (0)
 #define	md_setle_reg(inst,reg)	\
 			do { (inst) = _md_ppc_setcc \
-					((inst), (reg), PPC_CC_LE, PPC_CC_GT); } while (0)
+					((inst), (reg), PPC_CC_LE); } while (0)
 #define	md_setgt_reg(inst,reg)	\
 			do { (inst) = _md_ppc_setcc \
-					((inst), (reg), PPC_CC_GT, PPC_CC_LE); } while (0)
+					((inst), (reg), PPC_CC_GT); } while (0)
 #define	md_setge_reg(inst,reg)	\
 			do { (inst) = _md_ppc_setcc \
-					((inst), (reg), PPC_CC_GE, PPC_CC_LT); } while (0)
+					((inst), (reg), PPC_CC_GE); } while (0)
 
 /*
  * Set a register to -1, 0, or 1 based on comparing two values.
@@ -588,27 +640,27 @@ extern md_inst_ptr _md_ppc_setcmp(md_inst_ptr inst, int dreg);
  * jump offset will be filled in by a later "md_patch" call.
  */
 #define	md_branch_eq(inst)	\
-			ppc_branch_imm((inst), PPC_CC_EQ, 0)
+			ppc_branch((inst), PPC_CC_EQ, 0)
 #define	md_branch_ne(inst)	\
-			ppc_branch_imm((inst), PPC_CC_NE, 0)
+			ppc_branch((inst), PPC_CC_NE, 0)
 #define	md_branch_lt(inst)	\
-			ppc_branch_imm((inst), PPC_CC_LT, 0)
+			ppc_branch((inst), PPC_CC_LT, 0)
 #define	md_branch_le(inst)	\
-			ppc_branch_imm((inst), PPC_CC_LE, 0)
+			ppc_branch((inst), PPC_CC_LE, 0)
 #define	md_branch_gt(inst)	\
-			ppc_branch_imm((inst), PPC_CC_GT, 0)
+			ppc_branch((inst), PPC_CC_GT, 0)
 #define	md_branch_ge(inst)	\
-			ppc_branch_imm((inst), PPC_CC_GE, 0)
+			ppc_branch((inst), PPC_CC_GE, 0)
 #define	md_branch_lt_un(inst)	\
-			ppc_branch_imm((inst), PPC_CC_LT, 0)
+			ppc_branch((inst), PPC_CC_LT, 0)
 #define	md_branch_le_un(inst)	\
-			ppc_branch_imm((inst), PPC_CC_LE, 0)
+			ppc_branch((inst), PPC_CC_LE, 0)
 #define	md_branch_gt_un(inst)	\
-			ppc_branch_imm((inst), PPC_CC_GT, 0)
+			ppc_branch((inst), PPC_CC_GT, 0)
 #define	md_branch_ge_un(inst)	\
-			ppc_branch_imm((inst), PPC_CC_GE, 0)
+			ppc_branch((inst), PPC_CC_GE, 0)
 #define	md_branch_cc(inst,cond)	\
-			ppc_branch_imm((inst), (cond) & ~16, 0)
+			ppc_branch((inst), (cond) & ~16, 0)
 
 /*
  * Specific condition codes for "md_branch_cc".
@@ -631,7 +683,7 @@ extern md_inst_ptr _md_ppc_setcmp(md_inst_ptr inst, int dreg);
  * Back-patch a branch instruction at "patch" to branch to "inst".
  */
 #define	md_patch(patch,inst)	\
-			arm_patch((patch), (inst))
+			ppc_patch((patch), (inst))
 
 /*
  * Check an array bounds value.  "reg1" points to the array,
@@ -640,8 +692,12 @@ extern md_inst_ptr _md_ppc_setcmp(md_inst_ptr inst, int dreg);
  */
 #define	md_bounds_check(inst,reg1,reg2)	\
 			do { \
-				arm_load_advance((inst), ARM_WORK, (reg1)); \
-				arm_test_reg_reg((inst), ARM_CMP, (reg2), ARM_WORK); \
+				/* faking load_advance is easier for memindex code  \
+				   ppc_work = *(reg)++; (so to speak) \
+				*/ \
+				ppc_load_membase((inst), PPC_WORK, (reg1), 0);\
+				ppc_add_reg_imm((inst), (reg1), (reg1), sizeof(System_Array));\
+				ppc_cmp_reg_reg((inst), PPC_CMPL, (reg2), PPC_WORK); \
 			} while (0)
 
 /*
@@ -650,73 +706,73 @@ extern md_inst_ptr _md_ppc_setcmp(md_inst_ptr inst, int dreg);
  * "disp" if they advance the base pointer in "md_bounds_check".
  */
 #define	md_load_memindex_word_32(inst,reg,basereg,indexreg,disp)	\
-			arm_load_memindex((inst), (reg), (basereg), (indexreg))
+			ppc_load_memindex((inst), (reg), (basereg), (indexreg))
 
 /*
  * Load a native word value from an indexed array.
  */
 #define	md_load_memindex_word_native(inst,reg,basereg,indexreg,disp)	\
-			arm_load_memindex((inst), (reg), (basereg), (indexreg))
+			ppc_load_memindex((inst), (reg), (basereg), (indexreg))
 
 /*
  * Load a byte value from an indexed array.
  */
 #define	md_load_memindex_byte(inst,reg,basereg,indexreg,disp)	\
-			arm_load_memindex_byte((inst), (reg), (basereg), (indexreg))
+			ppc_load_memindex_byte((inst), (reg), (basereg), (indexreg))
 
 /*
  * Load a signed byte value from an indexed array.
  */
 #define	md_load_memindex_sbyte(inst,reg,basereg,indexreg,disp)	\
-			arm_load_memindex_sbyte((inst), (reg), (basereg), (indexreg))
+			ppc_load_memindex_sbyte((inst), (reg), (basereg), (indexreg))
 
 /*
  * Load a short value from an indexed array.
  */
 #define	md_load_memindex_short(inst,reg,basereg,indexreg,disp)	\
-			arm_load_memindex_short((inst), (reg), (basereg), (indexreg))
+			ppc_load_memindex_short((inst), (reg), (basereg), (indexreg))
 
 /*
  * Load an unsigned short value from an indexed array.
  */
 #define	md_load_memindex_ushort(inst,reg,basereg,indexreg,disp)	\
-			arm_load_memindex_ushort((inst), (reg), (basereg), (indexreg))
+			ppc_load_memindex_ushort((inst), (reg), (basereg), (indexreg))
 
 /*
  * Store a 32-bit word value into an indexed array.
  */
 #define	md_store_memindex_word_32(inst,reg,basereg,indexreg,disp)	\
-			arm_store_memindex((inst), (reg), (basereg), (indexreg))
+			ppc_store_memindex((inst), (reg), (basereg), (indexreg))
 
 /*
  * Store a native word value into an indexed array.
  */
 #define	md_store_memindex_word_native(inst,reg,basereg,indexreg,disp)	\
-			arm_store_memindex((inst), (reg), (basereg), (indexreg))
+			ppc_store_memindex((inst), (reg), (basereg), (indexreg))
 
 /*
  * Store a byte value into an indexed array.
  */
 #define	md_store_memindex_byte(inst,reg,basereg,indexreg,disp)	\
-			arm_store_memindex_byte((inst), (reg), (basereg), (indexreg))
+			ppc_store_memindex_byte((inst), (reg), (basereg), (indexreg))
 
 /*
  * Store a signed byte value into an indexed array.
  */
 #define	md_store_memindex_sbyte(inst,reg,basereg,indexreg,disp)	\
-			arm_store_memindex_sbyte((inst), (reg), (basereg), (indexreg))
+			ppc_store_memindex_sbyte((inst), (reg), (basereg), (indexreg))
 
 /*
  * Store a short value into an indexed array.
  */
 #define	md_store_memindex_short(inst,reg,basereg,indexreg,disp)	\
-			arm_store_memindex_short((inst), (reg), (basereg), (indexreg))
+			ppc_store_memindex_short((inst), (reg), (basereg), (indexreg))
 
 /*
  * Store an unsigned short value into an indexed array.
  */
 #define	md_store_memindex_ushort(inst,reg,basereg,indexreg,disp)	\
-			arm_store_memindex_ushort((inst), (reg), (basereg), (indexreg))
+			ppc_store_memindex_ushort((inst), (reg), (basereg), (indexreg))
 
 #ifdef	__cplusplus
 };
