@@ -20,6 +20,9 @@
 
 #include "engine.h"
 #include "lib_defs.h"
+#if defined(IL_WIN32_PLATFORM)
+#include <objbase.h>
+#endif
 
 #if defined(HAVE_LIBFFI)
 
@@ -49,6 +52,22 @@ int _ILCVMCanUseRawCalls(ILMethod *method, int isInternal);
 extern	"C" {
 #endif
 
+#if 0
+/* Blindly freeing return values is extremely dangerous.  Will fix this
+   problem when PInvoke is rewritten from scratch -- Rhys */
+#if defined(IL_WIN32_PLATFORM)
+	/* MS.NET frees native strings using the COM allocator 
+	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpconmemorymanagement.asp
+	*/
+	#define ILFreeNativeString(str) CoTaskMemFree(str)
+#else
+	/* Use the C allocator for other platforms */
+	#define ILFreeNativeString(str) free(str)
+#endif
+#else
+	#define ILFreeNativeString(str) do { ; } while (0)
+#endif
+
 /*
  * Structure type for passing typed references on the stack.
  */
@@ -515,6 +534,10 @@ static int PackDelegateParams(ILExecThread *thread, ILMethod *method,
 					if(strValue)
 					{
 						stacktop->ptrValue = ILStringCreate(thread, strValue);
+
+						/* Free the native string */
+						ILFreeNativeString(strValue);
+
 						if(!(stacktop->ptrValue))
 						{
 							return 1;
@@ -538,6 +561,10 @@ static int PackDelegateParams(ILExecThread *thread, ILMethod *method,
 					{
 						stacktop->ptrValue =
 							ILStringCreateUTF8(thread, strValue);
+
+						/* Free the native string */
+						ILFreeNativeString(strValue);
+
 						if(!(stacktop->ptrValue))
 						{
 							return 1;
@@ -561,6 +588,10 @@ static int PackDelegateParams(ILExecThread *thread, ILMethod *method,
 					{
 						stacktop->ptrValue =
 							ILStringWCreate(thread, (ILUInt16 *)strValue);
+
+						/* Free the native string */
+						ILFreeNativeString(strValue);
+					
 						if(!(stacktop->ptrValue))
 						{
 							return 1;
