@@ -21,6 +21,7 @@
 #ifndef	_PPC_CODEGEN_H
 #define	_PPC_CODEGEN_H
 
+#include <stdio.h>
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -62,6 +63,13 @@ typedef enum
 	PPC_R29  = 29,
 	PPC_R30  = 30,
 	PPC_R31  = 31,
+	
+	/* PPC Special registers */
+	PPC_XER  = 32,  
+	PPC_LR   = 256,
+	PPC_CTR  = 288,
+
+	/* redefinitions */
 	PPC_SP   = PPC_R1,
 	PPC_WORK = PPC_R12,
 
@@ -114,6 +122,7 @@ typedef enum
 {
 	PPC_CC_BRFALSE	= (4 << 5),				/* Branch if condition false */
 	PPC_CC_BRTRUE	= (12 << 5),			/* Branch if condition true */
+	PPC_CC_ALWAYS   = (20 << 5),			/* Uncontional Branch */
 	PPC_CC_EQ		= PPC_CC_BRTRUE | 2,	/* Equal */
 	PPC_CC_NE		= PPC_CC_BRFALSE | 2,	/* Not equal */
 	PPC_CC_LT		= PPC_CC_BRTRUE | 0,	/* Less than */
@@ -131,6 +140,7 @@ typedef enum
 	PPC_ADD		= 266,			/* Add */
 	PPC_ADDC	= 10,			/* Add with carry out */
 	PPC_ADDE	= 138,			/* Add with extended carry in */
+	PPC_ADDIC	= 21,			/* Add immediate with carry (ds_imm) */ 
 	PPC_SUBF	= 40,			/* Subtract from */
 	PPC_SUBFC	= 8,			/* Subtract from with carry out */
 	PPC_SUBFE	= 136,			/* Subtract from with extended carry in */
@@ -144,6 +154,7 @@ typedef enum
 	PPC_OR		= 444,			/* Or (sds) */
 	PPC_ORC		= 412,			/* Or with complement (sds) */
 	PPC_XOR		= 316,			/* Xor (sds) */
+	PPC_XORI	= 26,			/* Xor (imm) */
 	PPC_CMP		= 0,			/* Signed compare */
 	PPC_CMPL	= 32,			/* Unsigned (logical) compare */
 	PPC_NEG		= 104,			/* Negate */
@@ -152,14 +163,41 @@ typedef enum
 	PPC_SR		= 536,			/* Shift right (sds) */
 	PPC_EXTSB	= 954,			/* Extend signed byte */
 	PPC_EXTSH	= 922,			/* Extend signed half word */
+	PPC_CNTLZ   = 26,			/* Count leading zeros (ds) */
 
 } PPC_OP;
 
+/*
+ * FPU operations.
+ */
+typedef enum
+{
+	PPC_FADD	= 21,			/* Add (dab) */
+	PPC_FSUB	= 20,			/* Sub (dab) */
+	PPC_FMUL	= 25,			/* Mul (dac) */
+	PPC_FDIV	= 18,			/* div (dab) */
+	PPC_FCMPU	= 0, 			/* fcmpu (ab) */
+	PPC_FCMPO	= 32,			/* fcmpo (ab) */
+	PPC_FNEG	= 40,			/* fneg (db) */
+} PPC_FLOP;
+
 /*
  * Type for instruction pointers (word-based, not byte-based).
  */
 typedef unsigned int *ppc_inst_ptr;
 
+/* 
+ * Trap TODO code while executing
+ */
+#define TODO_trap(inst) \
+	do {\
+		*(inst)++ = ((31 << 26)  \
+						| (31 << 21) \
+						| (0 << 16) \
+						| (0 << 11) \
+						| (4 << 1)); \
+	} while(0)
+
 /*
  * Perform an arithmetic or logical operation.
  */
@@ -194,6 +232,23 @@ typedef unsigned int *ppc_inst_ptr;
 							 (((unsigned int)(opc)) << 1)); \
 			} while (0)
 
+#define ppc_alu_reg_imm(inst, opc, dreg, sreg, imm) \
+		do {\
+			int __value = (int)(imm);\
+			if(__value >= -0x8000 && __value <= 0x7FFF) \
+			{\
+				*(inst)++ = ((((unsigned int)(opc))<<	26) \
+								| (((unsigned int)(dreg)) << 21) \
+								| (((unsigned int)(sreg)) << 16) \
+								| (((unsigned int)(__value)) & 0xFFFF));\
+			}\
+			else\
+			{\
+				TODO_trap(inst);\
+			}\
+		} while(0)
+
+
 /*
  * Perform an arithmetic or logical operation and also set the
  * condition codes.
@@ -246,42 +301,42 @@ typedef unsigned int *ppc_inst_ptr;
  */
 #define	ppc_add_reg_imm(inst,dreg,sreg,imm)	\
 			do { \
-				int __value = (int)(imm); \
-				if(__value >= -0x8000 && __value <= 0x7FFF) \
+				int __add_value = (int)(imm); \
+				if(__add_value >= -0x8000 && __add_value <= 0x7FFF) \
 				{ \
 					*(inst)++ = ((14 << 26) | \
 								 (((unsigned int)(dreg)) << 21) | \
 								 (((unsigned int)(sreg)) << 16) | \
-								 (((unsigned int)(__value)) & 0xFFFF)); \
+								 (((unsigned int)(__add_value)) & 0xFFFF)); \
 				} \
-				else if((__value & 0xFFFF) == 0) \
+				else if((__add_value & 0xFFFF) == 0) \
 				{ \
 					*(inst)++ = ((15 << 26) | \
 								 (((unsigned int)(dreg)) << 21) | \
 								 (((unsigned int)(sreg)) << 16) | \
-								 (((unsigned int)(__value >> 16)) & 0xFFFF)); \
+								 (((unsigned int)(__add_value >> 16)) & 0xFFFF)); \
 				} \
-				else if((__value & 0x8000) == 0) \
+				else if((__add_value & 0x8000) == 0) \
 				{ \
 					*(inst)++ = ((14 << 26) | \
 								 (((unsigned int)PPC_WORK) << 21) | \
 								 (((unsigned int)(sreg)) << 16) | \
-								 (((unsigned int)(__value)) & 0xFFFF)); \
+								 (((unsigned int)(__add_value)) & 0xFFFF)); \
 					*(inst)++ = ((15 << 26) | \
 								 (((unsigned int)(dreg)) << 21) | \
 								 (((unsigned int)PPC_WORK) << 16) | \
-								 (((unsigned int)(__value >> 16)) & 0xFFFF)); \
+								 (((unsigned int)(__add_value >> 16)) & 0xFFFF)); \
 				} \
 				else \
 				{ \
 					*(inst)++ = ((14 << 26) | \
 								 (((unsigned int)PPC_WORK) << 21) | \
 								 (((unsigned int)(sreg)) << 16) | \
-								 (((unsigned int)(__value)) & 0xFFFF)); \
+								 (((unsigned int)(__add_value)) & 0xFFFF)); \
 					*(inst)++ = ((15 << 26) | \
 								 (((unsigned int)(dreg)) << 21) | \
 								 (((unsigned int)PPC_WORK) << 16) | \
-								 (((unsigned int)((__value >> 16) + 1)) \
+								 (((unsigned int)((__add_value >> 16) + 1)) \
 								 		& 0xFFFF)); \
 				} \
 			} while (0)
@@ -357,58 +412,145 @@ typedef unsigned int *ppc_inst_ptr;
  */
 #define	ppc_cmp_reg_imm(inst,opc,reg,imm)	\
 			do { \
-				int __value = (int)(imm); \
+				int __cmp_value = (int)(imm); \
 				if((opc) == PPC_CMP) \
 				{ \
-					if(__value >= -0x8000 && __value <= 0x7FFF) \
+					if(__cmp_value >= -0x8000 && __cmp_value <= 0x7FFF) \
 					{ \
 						*(inst)++ = ((11 << 26) | \
 									 (((unsigned int)(reg)) << 16) | \
-									 (((unsigned int)(__value)) & 0xFFFF)); \
+									 (((unsigned int)(__cmp_value)) & 0xFFFF)); \
 					} \
 					else \
 					{ \
-						ppc_mov_reg_imm((inst), PPC_WORK, __value); \
+						ppc_mov_reg_imm((inst), PPC_WORK, __cmp_value); \
 						ppc_cmp_reg_reg((inst), PPC_CMP, (reg), PPC_WORK); \
 					} \
 				} \
 				else \
 				{ \
-					if(__value >= 0 && __value <= 0xFFFF) \
+					if(__cmp_value >= 0 && __cmp_value <= 0xFFFF) \
 					{ \
 						*(inst)++ = ((10 << 26) | \
 									 (((unsigned int)(reg)) << 16) | \
-									  ((unsigned int)(__value))); \
+									  ((unsigned int)(__cmp_value))); \
 					} \
 					else \
 					{ \
-						ppc_mov_reg_imm((inst), PPC_WORK, __value); \
+						ppc_mov_reg_imm((inst), PPC_WORK, __cmp_value); \
 						ppc_cmp_reg_reg((inst), PPC_CMPL, (reg), PPC_WORK); \
 					} \
 				} \
 			} while (0)
 
+
+#define ppc_fpu_reg_dab(inst, f64, opc, dreg, freg1, freg2) \
+	do {\
+		*(inst)++ = (((f64 ? 63 : 59) << 26) \
+						| (((unsigned int)(dreg)) << 21) \
+						| (((unsigned int)(freg1)) << 16) \
+						| (((unsigned int)(freg2)) << 11) \
+						| (((unsigned int)(opc)) << 1));\
+	}while(0)
+
+#define ppc_fpu_reg_dac(inst, f64, opc, dreg, freg1, freg2) \
+	do {\
+		*(inst)++ = (((f64 ? 63 : 59) << 26) \
+						| (((unsigned int)(dreg)) << 21) \
+						| (((unsigned int)(freg1)) << 16) \
+						| (((unsigned int)(freg2)) << 6) \
+						| (((unsigned int)(opc)) << 1));\
+	}while(0)
+
+#define ppc_fpu_fsrp(inst, dreg, freg1) \
+	do {\
+		*(inst)++ = ((63 << 26) \
+						| (((unsigned int)(dreg)) << 21) \
+						| (((unsigned int)(freg1)) << 11) \
+						| (((unsigned int)(12)) << 1));\
+	}while(0)
+
 /*
  * Jump to a particular address.
  */
-#define	ppc_jump(inst,target)	\
+#define	ppc_jump(inst, target)	\
 			do { \
-				/* TODO */ \
+				int __ld_offset = (target == 0) ? 0 : \
+									(ppc_inst_ptr)(target) - ((inst) + 1);\
+				if(__ld_offset >= -0x8000 && __ld_offset <= 0x7FFF)\
+				{\
+					*(inst)++ = ((18 << 26) | \
+								((((unsigned int)__ld_offset) & 0xFFFF) << 2));\
+				}\
+				else\
+				{\
+					TODO_trap(inst);\
+				}\
 			} while (0)
 
+/*
+ * Jump to address pointed by basereg
+ */
+#define ppc_jump_reg(inst, basereg) \
+			do { \
+					/* mtctr , bcctr */ \
+					*(inst)++ = ((31 << 26) \
+								| (((unsigned int)(basereg)) << 21)	\
+								| (((unsigned int)(PPC_CTR)) << 11) \
+								| (467 << 1)) ;\
+					*(inst)++ = ((19 << 26) \
+								| (((unsigned int)(PPC_CC_ALWAYS)) << 16) \
+								| (528 << 1)) ;\
+			} while(0);
+
 /*
  * Jump to a particular address based on a condition.
  */
 #define	ppc_branch(inst,cond,target)	\
 			do { \
-				/* TODO */ \
+				int __ld_offset = (target == 0 ) ? 0 : \
+								(ppc_inst_ptr)(target) - ((inst) + 1);\
+				if(__ld_offset >= -0x8000 && __ld_offset <= 0x7FFF)\
+				{\
+					*(inst)++ = ((16 << 26) \
+								| (((unsigned int)(cond)) << 16) \
+								| ((((unsigned int)__ld_offset) & 0xFFFF) << 2));\
+				}\
+				else\
+				{\
+					TODO_trap(inst);\
+				}\
 			} while (0)
 
+/*
+ * Back-patch a branch instruction
+ */
+#define ppc_patch(patch, inst) \
+			do {\
+				 ppc_inst_ptr __patch = (ppc_inst_ptr)(patch);\
+				 int __ld_offset = (inst) - (__patch);\
+				 if(__ld_offset < -0x8000 && __ld_offset > 0x7FFF)\
+				 {\
+				 	TODO_trap(inst);\
+				 }\
+				 if(((*__patch) & (63 << 26)) == (16 << 26)) \
+				 {\
+				 	(*__patch) = (*__patch) & ((~0xFFFF) | 3);\
+				 	(*__patch) = ((*__patch) | \
+								((((unsigned int)__ld_offset) & 0xFFFF) << 2));\
+				 }\
+				 else if(((*__patch) & (63 << 26)) == (18 << 26))\
+				 {\
+				 	(*__patch) = (*__patch) & ((~0x3FFFFFF) | 3);\
+				 	(*__patch) = ((*__patch) | \
+								((((unsigned int)__ld_offset) & 0xFFFFFF) << 2));\
+				 }\
+			} while(0);
+
 /*
  * Load various sized values from a memory location defined by a base
  * register and an offset.
  */
-extern ppc_inst_ptr _md_ppc_get_ea(ppc_inst_ptr inst, int basereg, int offset);
 #define	ppc_membase_common(inst,reg,basereg,offset,opc)	\
 			do { \
 				int __ld_offset = (int)(offset); \
@@ -421,7 +563,9 @@ extern ppc_inst_ptr _md_ppc_get_ea(ppc_inst_ptr inst, int basereg, int offset);
 				} \
 				else \
 				{ \
-					(inst) = _md_ppc_get_ea((inst), (basereg), __ld_offset); \
+					ppc_mov_reg_imm(inst, PPC_WORK, (__ld_offset));\
+					ppc_alu_reg_dss(inst, PPC_ADD, \
+									PPC_WORK, PPC_WORK, (basereg));\
 					*(inst)++ = (((opc) << 26) | \
 								 (((unsigned int)(reg)) << 21) | \
 								 (((unsigned int)PPC_WORK) << 16)); \
@@ -441,6 +585,10 @@ extern ppc_inst_ptr _md_ppc_get_ea(ppc_inst_ptr inst, int basereg, int offset);
 #define	ppc_load_membase_ushort(inst,reg,basereg,offset)	\
 			ppc_membase_common((inst), (reg), (basereg), (offset), 40)
 
+#define	ppc_load_membase_float_32(inst,freg,basereg,offset)	\
+			ppc_membase_common((inst), (freg), (basereg), (offset), 48)
+#define	ppc_load_membase_float_64(inst,freg,basereg,offset)	\
+			ppc_membase_common((inst), (freg), (basereg), (offset), 50)
 /*
  * Store values of various sizes to a memory location defined by a
  * base register plus an offset.
@@ -456,6 +604,65 @@ extern ppc_inst_ptr _md_ppc_get_ea(ppc_inst_ptr inst, int basereg, int offset);
 #define	ppc_store_membase_ushort(inst,reg,basereg,offset)	\
 			ppc_membase_common((inst), (reg), (basereg), (offset), 44)
 
+#define	ppc_store_membase_float_32(inst,freg,basereg,offset)	\
+			ppc_membase_common((inst), (freg), (basereg), (offset), 52)
+#define	ppc_store_membase_float_64(inst,freg,basereg,offset)	\
+			ppc_membase_common((inst), (freg), (basereg), (offset), 54)
+/*
+ * Load various sized values from a memory location defined by a base
+ * register and an index register.
+ */
+#define	ppc_memindex_common(inst,reg,basereg,indexreg, shift, opc)	\
+			do { \
+				if((shift))\
+				{ \
+					/* slwi PPC_WORK, indexreg, shift \
+					(ie multiply by pointer index increment) */\
+					ppc_alu_rlwinm(inst, PPC_WORK, (indexreg), shift, 0, 31-shift);\
+					*(inst)++ = ((31 << 26) \
+								| (((unsigned int)(reg)) << 21) \
+								| (((unsigned int)(basereg)) << 16) \
+								| (((unsigned int)(PPC_WORK)) << 11)\
+								| (((unsigned int)(opc)) << 1));\
+				} \
+				else \
+				{ \
+					*(inst)++ = ((31 << 26) \
+								| (((unsigned int)(reg)) << 21) \
+								| (((unsigned int)(basereg)) << 16) \
+								| (((unsigned int)(indexreg)) << 11)\
+								| (((unsigned int)(opc)) << 1));\
+				} \
+			} while (0)
+
+#define	ppc_load_memindex(inst,reg,basereg,indexreg)	/* lwzx */ \
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 2, 23) 
+#define	ppc_load_memindex_byte(inst,reg,basereg,indexreg)	/* lbzx */ \
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 0, 87)
+#define	ppc_load_memindex_sbyte(inst,reg,basereg,indexreg)	\
+			do { \
+				ppc_memindex_common((inst), (reg), (basereg), (indexreg), 0, 87); \
+				ppc_alu_reg_sd((inst), PPC_EXTSB, (reg), (reg)); \
+			} while (0)
+#define	ppc_load_memindex_short(inst,reg,basereg,indexreg)	/* lhax */\
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 1, 343)
+#define	ppc_load_memindex_ushort(inst,reg,basereg,indexreg)	/* lhzx */\
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 1, 279)
+
+/*
+ * Store values of various sizes to a memory location defined by a
+ * base register plus an index register.
+ */
+#define	ppc_store_memindex(inst,reg,basereg,indexreg)	/* stwx */\
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 2, 151)
+#define	ppc_store_memindex_byte(inst,reg,basereg,indexreg)	/*  stbx */\
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 0, 215)
+#define	ppc_store_memindex_sbyte(inst,reg,basereg,indexreg)	\
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 0, 215)
+#define	ppc_store_memindex_short(inst,reg,basereg,indexreg) /* sthx */	\
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 1, 407)
+#define	ppc_store_memindex_ushort(inst,reg,basereg,indexreg) /* sthx */	\
+			ppc_memindex_common((inst), (reg), (basereg), (indexreg), 1, 407)
 /*
  * Push the contents of a word register onto the stack.
  */
@@ -471,6 +678,39 @@ extern ppc_inst_ptr _md_ppc_get_ea(ppc_inst_ptr inst, int basereg, int offset);
 				ppc_add_reg_imm((inst), PPC_SP, PPC_SP, 4); \
 			} while (0)
 
+/*
+ * Rotate Left Word Immediate then AND with Mask (ppc direct) 
+ */
+
+#define ppc_alu_rlwinm(inst, dreg, reg, n, mb, me) \
+		do {\
+			*(inst)++ = ((21 << 26) \
+							| (((unsigned int)(reg)) << 21) \
+							| (((unsigned int)(dreg)) << 16) \
+							| (((unsigned int)(n)) <<  11) \
+							| (((unsigned int)(mb)) << 6) \
+							| (((unsigned int)(me)) << 1)); \
+		} while(0)
+
+/*
+ * Add immediate with carry 
+ */
+#define ppc_alu_addc_imm(inst, dreg, sreg, imm) \
+		do {\
+			int __value = (int)(imm);\
+			if(__value >= -0x8000 && __value <= 0x7FFF) \
+			{\
+				*(inst)++ = ((21 <<	26) \
+								| (((unsigned int)(dreg)) << 21) \
+								| (((unsigned int)(sreg)) << 16) \
+								| (((unsigned int)(__value)) & 0xFFFF));\
+			}\
+			else\
+			{\
+				TODO_trap(inst);\
+			}\
+		} while(0)
+
 #ifdef __cplusplus
 };
 #endif
