@@ -566,6 +566,20 @@ static int IsSubClass(ILType *type, ILClass *classInfo)
 	}
 }
 
+/*
+ * Push the appropriate synchronization object for a synchronized method.
+ */
+#define PUSH_SYNC_OBJECT() \
+	if (isStatic) \
+	{ \
+		ILCoderPushToken(coder, (ILProgramItem *)ILMethod_Owner(method)); \
+		ILCoderCallInlineable(coder, IL_INLINEMETHOD_TYPE_FROM_HANDLE, 0); \
+	} \
+	else \
+	{ \
+		ILCoderLoadArg(coder, 0, ILType_FromClass(ILMethod_Owner(method))); \
+	}
+
 /*
  * Bailout routines for various kinds of verification failure.
  */
@@ -602,9 +616,9 @@ static int IsSubClass(ILType *type, ILClass *classInfo)
 #include "verify_conv.c"
 #include "verify_stack.c"
 #include "verify_ptr.c"
-#include "verify_call.c"
 #include "verify_obj.c"
 #include "verify_branch.c"
+#include "verify_call.c"
 #include "verify_except.c"
 #include "verify_ann.c"
 #undef IL_VERIFY_GLOBALS
@@ -619,6 +633,7 @@ int _ILVerify(ILCoder *coder, unsigned char **start, ILMethod *method,
 	int result;
 	unsigned opcode;
 	ILUInt32 insnSize;
+	int isStatic, isSynchronized;
 	int insnType;
 	ILUInt32 offset = 0;
 	ILEngineStackItem *stack;
@@ -642,6 +657,10 @@ int _ILVerify(ILCoder *coder, unsigned char **start, ILMethod *method,
 	ILException *exception, *currentException;
 	int hasRethrow;
 	int tailCall = 0;
+	int tryInlineType;
+	int coderFlags;
+	unsigned int tryInlineOpcode;
+	unsigned char *tryInlinePc;
 #ifdef IL_CONFIG_DEBUG_LINES
 	int haveDebug = ILDebugPresent(ILProgramItem_Image(method));
 #else
@@ -656,8 +675,8 @@ int _ILVerify(ILCoder *coder, unsigned char **start, ILMethod *method,
 #include "verify_conv.c"
 #include "verify_stack.c"
 #include "verify_ptr.c"
-#include "verify_call.c"
 #include "verify_obj.c"
+#include "verify_call.c"
 #include "verify_branch.c"
 #include "verify_except.c"
 #include "verify_ann.c"
@@ -669,6 +688,10 @@ int _ILVerify(ILCoder *coder, unsigned char **start, ILMethod *method,
 		return 0;
 	}
 
+	coderFlags = ILCoderGetFlags(coder);
+	isStatic = ILMethod_IsStatic(method);
+	isSynchronized = ILMethod_IsSynchronized(method);
+		
 restart:
 	result = 0;
 	labelList = 0;
@@ -1001,7 +1024,7 @@ restart:
 	}
 
 	/* Set up for exception handling if necessary */
-	if(exceptions)
+	if(exceptions || isSynchronized)
 	{
 		ILCoderSetupExceptions(coder, exceptions, hasRethrow);
 	}
@@ -1010,6 +1033,14 @@ restart:
 	pc = code->code;
 	len = code->codeLen;
 	lastWasJump = 0;
+
+	/* If the method is synchronized then generate the Monitor.Enter call */
+	if (isSynchronized)
+	{
+		PUSH_SYNC_OBJECT();
+		ILCoderCallInlineable(coder, IL_INLINEMETHOD_MONITOR_ENTER, 0);
+	}
+
 	while(len > 0)
 	{
 		/* Fetch the instruction information block */
@@ -1081,9 +1112,9 @@ restart:
 #include "verify_conv.c"
 #include "verify_stack.c"
 #include "verify_ptr.c"
-#include "verify_call.c"
 #include "verify_obj.c"
 #include "verify_branch.c"
+#include "verify_call.c"
 #include "verify_except.c"
 #include "verify_ann.c"
 #undef IL_VERIFY_CODE
@@ -1106,7 +1137,7 @@ restart:
 	ILCoderMarkEnd(coder);
 
 	/* Output the exception handler table, if necessary */
-	if(exceptions != 0)
+	if(exceptions != 0 || isSynchronized)
 	{
 		OutputExceptionTable(coder, method, exceptions, hasRethrow);
 	}
