@@ -117,6 +117,12 @@ static ILCmdLineOption const options[] = {
 	{"-H", 'H', 1,
 		"-fhash-algorithm=name       or -H name",
 		"Specify the algorithm to use to hash files (SHA1 or MD5)"},
+	{"-V", 'V', 1,
+		"-fmetadata-version=version  or -V version",
+		"Specify the metadata version to embed in the assembly."},
+	{"-u", 'u', 1,
+		"-fculture=name              or -u name",
+		"Specify the resource culture (e.g. \"en-US\")."},
 	{"-3", '3', 0,
 		"-m32bit-only                or -3",
 		"The resulting output file can only be used on 32-bit systems."},
@@ -145,7 +151,7 @@ static ILCmdLineOption const options[] = {
 	{"/comp*", '?', 1, 0, 0},		/* "/company:text" */
 	{"/config*", '?', 1, 0, 0},		/* "/configuration:text" */
 	{"/copy*", '?', 1, 0, 0},		/* "/copyright:text" */
-	{"/c*", '?', 1, 0, 0},			/* "/culture:text" */
+	{"/c*", 'u', 1, 0, 0},			/* "/culture:text" */
 	{"/delay*", '?', 0, 0, 0},		/* "/delaysign[+|-]" */
 	{"/descr*", '?', 1, 0, 0},		/* "/description:text" */
 	{"/embed*", 'Q', 1, 0, 0},		/* "/embedresource:filename" */
@@ -177,7 +183,6 @@ static ILCmdLineOption const options[] = {
 static void usage(const char *progname);
 static void version(void);
 static void outOfMemory(void);
-static int parseVersion(ILUInt16 *version, const char *str);
 static int addLibrary(ILLinker *linker, const char *filename);
 static int addResource(ILLinker *linker, const char *filename,
 					   FILE *stream, int privateResources, int isStdin);
@@ -205,6 +210,8 @@ int ILLinkerMain(int argc, char *argv[])
 	const char *publicKey = NULL;
 	char *entryPoint = NULL;
 	int hashAlgorithm = IL_META_HASHALG_SHA1;
+	char *metadataVersion = NULL;
+	char *culture = NULL;
 	int len;
 	char **libraries;
 	int numLibraries = 0;
@@ -346,7 +353,7 @@ int ILLinkerMain(int argc, char *argv[])
 
 			case 'A':
 			{
-				if(!parseVersion(assemblyVersion, param))
+				if(!ILLinkerParseVersion(assemblyVersion, param))
 				{
 					fprintf(stderr,
 							"%s: `%s' is not a valid assembly version\n",
@@ -356,6 +363,18 @@ int ILLinkerMain(int argc, char *argv[])
 			}
 			break;
 
+			case 'V':
+			{
+				metadataVersion = param;
+			}
+			break;
+
+			case 'u':
+			{
+				culture = param;
+			}
+			break;
+
 			case 'M':
 			{
 				moduleName = param;
@@ -518,7 +537,7 @@ int ILLinkerMain(int argc, char *argv[])
 				}
 				else if(!strncmp(param, "assembly-version=", 17))
 				{
-					if(!parseVersion(assemblyVersion, param + 17))
+					if(!ILLinkerParseVersion(assemblyVersion, param + 17))
 					{
 						fprintf(stderr,
 								"%s: `%s' is not a valid assembly version\n",
@@ -534,6 +553,10 @@ int ILLinkerMain(int argc, char *argv[])
 				{
 					moduleName = param + 12;
 				}
+				else if(!strncmp(param, "culture=", 8))
+				{
+					culture = param + 8;
+				}
 				else if(!strncmp(param, "resources=", 10))
 				{
 					resources[numResources] = param + 10;
@@ -546,6 +569,10 @@ int ILLinkerMain(int argc, char *argv[])
 					param += 15;
 					goto parseHashAlg;
 				}
+				else if(!strncmp(param, "metadata-version=", 17))
+				{
+					metadataVersion = param + 17;
+				}
 				else if(!strncmp(param, "win32res=", 9))
 				{
 					win32Res = param + 9;
@@ -787,6 +814,15 @@ int ILLinkerMain(int argc, char *argv[])
 		errors |= addLibrary(linker, stdLibrary);
 	}
 
+	/* Set the metadata version in the assembly's header */
+	ILLinkerSetMetadataVersion(linker, metadataVersion, stdLibrary);
+
+	/* Set the culture name on the assembly */
+	if(culture)
+	{
+		ILLinkerSetCulture(linker, culture);
+	}
+
 	/* Process the input files that aren't libraries */
 	sawStdin = 0;
 	firstFile = 1;
@@ -980,42 +1016,6 @@ static void outOfMemory(void)
 	exit(1);
 }
 
-/*
- * Parse an assembly version string.
- */
-static int parseVersion(ILUInt16 *version, const char *str)
-{
-	int posn;
-	ILUInt32 value;
-	for(posn = 0; posn < 4; ++posn)
-	{
-		if(*str < '0' || *str > '9')
-		{
-			return 0;
-		}
-		value = (ILUInt32)(*str++ - '0');
-		while(*str >= '0' && *str <= '9')
-		{
-			value = value * ((ILUInt32)10) + (ILUInt32)(*str++ - '0');
-			if(value >= (ILUInt32)0x10000)
-			{
-				return 0;
-			}
-		}
-		version[posn] = (ILUInt16)value;
-		if(posn == 3)
-		{
-			break;
-		}
-		if(*str != ':' && *str != '.')
-		{
-			return 0;
-		}
-		++str;
-	}
-	return (*str == '\0');
-}
-
 /*
  * Add a library to a linker context.  Returns non-zero on error.
  */
