@@ -86,14 +86,14 @@ struct _tagILLibrary
 {
 	char		   *name;			/* Name of the library's assembly */
 	char           *filename;		/* Filename for the library's assembly */
+	char		   *moduleName;		/* Name of the library's main module */
 	ILUInt16		version[4];		/* Version of the library's assembly */
 	ILLibrary	   *altNames;		/* Alternative names for the library */
 	unsigned char  *publicKey;		/* Public key value for the library */
 	ILUInt32		publicKeyLen;	/* Length of the public key value */
 	ILHashTable    *classHash;		/* Hash table for class name lookup */
 	ILHashTable    *symbolHash;		/* Hash table for global symbol lookup */
-	int				memoryModel;	/* Memory model for the library */
-	int				modelFlags;		/* Memory model flags for the library */
+	int				isCImage;		/* Non-zero if a C image */
 	ILMemPool		pool;			/* Memory pool for symbol allocation */
 	ILContext      *context;		/* Context containing the library image */
 	ILImage        *image;			/* Image that corresponds to the library */
@@ -137,11 +137,10 @@ struct _tagILLinker
 	ILMethod       *entryPoint;		/* Current entry point that is set */
 	ILUInt32		dataLength;		/* Length of ".sdata" section */
 	ILUInt32		tlsLength;		/* Length of ".tls" section */
-	int				memoryModel;	/* Memory model of a C application */
-	int				modelFlags;		/* Alignment flags for the memory model */
+	int				isCLink;		/* Non-zero if linking a C image */
 	ILHashTable    *symbolHash;		/* Hash table for global symbol lookup */
 	ILMemPool		pool;			/* Memory pool for symbol allocation */
-	const char     *moduleName;		/* Name of the "<Module>" class */
+	char     	   *moduleName;		/* Name of the "<Module>" class */
 	ILClass        *moduleClass;	/* Reference to the "<Module>" class */
 	char		   *initTempFile;	/* Temporary object file for init/fini */
 
@@ -220,6 +219,17 @@ int _ILLinkerFindSymbol(ILLinker *linker, const char *name,
 						char **aliasFor, ILLibrary **library,
 						ILMember **memberRef);
 
+/*
+ * Update a global definition that was found.
+ */
+void _ILLinkerUpdateSymbol(ILLinker *linker, const char *name,
+						   ILMember *member);
+
+/*
+ * Determine if we have a definition for a specific symbol.
+ */
+int _ILLinkerHasSymbol(ILLinker *linker, const char *name);
+
 /*
  * Create the global symbol hash for the image being linked.
  */
@@ -329,6 +339,11 @@ ILClass *_ILLinkerModuleClass(ILLinker *linker);
  */
 int _ILLinkerIsModule(ILClass *classInfo);
 
+/*
+ * Determine if a class is marked with "OpenSystem.C.GlobalScope".
+ */
+int _ILLinkerIsGlobalScope(ILClass *classInfo);
+
 /*
  * Get a new name for a private class, that must be renamed
  * to prevent clashes with similar classes in other modules.
@@ -356,6 +371,14 @@ int _ILLinkerLibraryReplacement(ILLinker *linker, ILLibraryFind *find,
  */
 ILImage *_ILLinkerCreateInitFini(ILLinker *linker);
 
+/*
+ * Create an attribute on a particular program item.
+ */
+void _ILLinkerCreateAttribute(ILLinker *linker, ILProgramItem *item,
+							  const char *name, const char *namespace,
+							  ILType *arg1Type, ILType *arg2Type,
+							  void *data, int len);
+
 #ifdef	__cplusplus
 };
 #endif
