@@ -253,10 +253,16 @@ void ILAsmSplitName(const char *str, int len, const char **name,
 	}
 }
 
-void ILAsmBuildNewClass(const char *name, ILClass *parent, ILUInt32 attrs)
+void ILAsmBuildNewClass(const char *name, ILAsmParamInfo *genericParams,
+						ILClass *parent, ILUInt32 attrs)
 {
 	ILClass *info;
 	char uniqueName[64];
+	ILAsmParamInfo *nextGeneric;
+	ILUInt32 genericNum;
+	ILGenericPar *genPar;
+	ILProgramItem *constraint;
+	ILTypeSpec *spec;
 
 	/* Set the default parent to "System.Object" if necessary */
 	if(!parent &&
@@ -333,6 +339,45 @@ void ILAsmBuildNewClass(const char *name, ILClass *parent, ILUInt32 attrs)
 	/* Set the class attributes */
 	ILClassSetAttrs(info, ~((ILUInt32)0), attrs);
 
+	/* Add the formal generic parameters, if necessary */
+	genericNum = 0;
+	while(genericParams != 0)
+	{
+		nextGeneric = genericParams->next;
+		genPar = ILGenericParCreate
+				(ILAsmImage, 0, ILToProgramItem(info), genericNum);
+		if(!genPar)
+		{
+			ILAsmOutOfMemory();
+		}
+		if(!ILGenericParSetName(genPar, genericParams->name))
+		{
+			ILAsmOutOfMemory();
+		}
+		if(genericParams->type)
+		{
+			if(ILType_IsClass(genericParams->type) ||
+			   ILType_IsValueType(genericParams->type))
+			{
+				constraint =
+					ILToProgramItem(ILType_ToClass(genericParams->type));
+			}
+			else
+			{
+				spec = ILTypeSpecCreate(ILAsmImage, 0, genericParams->type);
+				if(!spec)
+				{
+					ILAsmOutOfMemory();
+				}
+				constraint = ILToProgramItem(spec);
+			}
+			ILGenericParSetConstraint(genPar, constraint);
+		}
+		ILFree(genericParams);
+		genericParams = nextGeneric;
+		++genericNum;
+	}
+
 	/* Add the previous class to the class stack */
 	if(classStackSize >= classStackMax)
 	{
@@ -764,24 +809,25 @@ ILToken ILAsmResolveMember(ILProgramItem *scope, const char *name,
 	}
 
 	/* Look for a name and signature match on a member */
-	member = 0;
-	while((member = ILClassNextMemberByKind(classInfo, member, kind)) != 0)
+	if((member = ILClassNextMemberMatch(classInfo, 0, kind, name, sig)) != 0)
 	{
-		if(!strcmp(ILMember_Name(member), name) &&
-		   ILTypeIdentical(ILMember_Signature(member), sig))
-		{
-			return ILMember_Token(member);
-		}
+		return ILMember_Token(member);
 	}
 
-	/* If the class is "complete", then we have already seen its
-	   definition and we cannot add anything else to it */
-	if(ILClassIsComplete(classInfo))
+	/* Ignore the "class complete" check if this is a vararg call site,
+	   because we can always add vararg calls to a complete class */
+	if(kind != IL_META_MEMBERKIND_METHOD ||
+	   (ILType_Kind(sig) & IL_TYPE_COMPLEX_METHOD_SENTINEL) == 0)
 	{
-		ILAsmPrintMessage(ILAsmFilename, ILAsmLineNum,
-						  "cannot resolve member `%s'", name);
-		ILAsmErrors = 1;
-		return 0;
+		/* If the class is "complete", then we have already seen its
+		   definition and we cannot add anything else to it */
+		if(ILClassIsComplete(classInfo))
+		{
+			ILAsmPrintMessage(ILAsmFilename, ILAsmLineNum,
+							  "cannot resolve member `%s'", name);
+			ILAsmErrors = 1;
+			return 0;
+		}
 	}
 
 	/* Generate a MemberRef token within the class, which will hopefully
@@ -817,37 +863,33 @@ ILMethod *ILAsmMethodCreate(ILClass *classInfo, const char *name,
 	ILMethod *method;
 
 	/* See if there is already a method with a matching signature */
-	method = 0;
-	while((method = (ILMethod *)ILClassNextMemberByKind
-			(classInfo, (ILMember *)method, IL_META_MEMBERKIND_METHOD)) != 0)
+	if((method = (ILMethod *)ILClassNextMemberMatch
+			(classInfo, (ILMember *)0,
+			 IL_META_MEMBERKIND_METHOD, name, sig)) != 0)
 	{
-		if(!strcmp(ILMethod_Name(method), name) &&
-		   ILTypeIdentical(ILMethod_Signature(method), sig))
+		/* We already have a definition */
+		if((ILMethod_Token(method) & IL_META_TOKEN_MASK)
+				== IL_META_TOKEN_METHOD_DEF &&
+		   ILMethod_RVA(method) != 0)
 		{
-			/* We already have a definition */
-			if((ILMethod_Token(method) & IL_META_TOKEN_MASK)
-					== IL_META_TOKEN_METHOD_DEF &&
-			   ILMethod_RVA(method) != 0)
-			{
-				/* This is a duplicate */
-				ILAsmPrintMessage(ILAsmFilename, ILAsmLineNum,
-								  "duplicate definition for `%s'", name);
-				ILAsmErrors = 1;
-			}
-			else
-			{
-				/* Convert the MemberRef into a MethodDef */
-				ILMethodSetCallConv(method, ILType_CallConv(sig));
-				ILMemberSetAttrs((ILMember *)method,
-								 ~((ILUInt32)0), attributes);
-				if((ILMethod_Token(method) & IL_META_TOKEN_MASK)
-						== IL_META_TOKEN_MEMBER_REF)
-				{
-					ILMethodNewToken(method);
-				}
-			}
-			return method;
+			/* This is a duplicate */
+			ILAsmPrintMessage(ILAsmFilename, ILAsmLineNum,
+							  "duplicate definition for `%s'", name);
+			ILAsmErrors = 1;
 		}
+		else
+		{
+			/* Convert the MemberRef into a MethodDef */
+			ILMethodSetCallConv(method, ILType_CallConv(sig));
+			ILMemberSetAttrs((ILMember *)method,
+							 ~((ILUInt32)0), attributes);
+			if((ILMethod_Token(method) & IL_META_TOKEN_MASK)
+					== IL_META_TOKEN_MEMBER_REF)
+			{
+				ILMethodNewToken(method);
+			}
+		}
+		return method;
 	}
 
 	/* Create a new method block */
@@ -867,36 +909,32 @@ ILField *ILAsmFieldCreate(ILClass *classInfo, const char *name,
 	ILField *field;
 
 	/* See if there is already a field with a matching signature */
-	field = 0;
-	while((field = (ILField *)ILClassNextMemberByKind
-			(classInfo, (ILMember *)field, IL_META_MEMBERKIND_FIELD)) != 0)
+	if((field = (ILField *)ILClassNextMemberMatch
+			(classInfo, (ILMember *)0,
+			 IL_META_MEMBERKIND_FIELD, name, sig)) != 0)
 	{
-		if(!strcmp(ILField_Name(field), name) &&
-		   ILTypeIdentical(ILField_Type(field), sig))
+		/* We already have a definition */
+		if((ILField_Token(field) & IL_META_TOKEN_MASK)
+				== IL_META_TOKEN_FIELD_DEF)
 		{
-			/* We already have a definition */
-			if((ILField_Token(field) & IL_META_TOKEN_MASK)
-					== IL_META_TOKEN_FIELD_DEF)
-			{
-				/* This is a duplicate */
-				ILAsmPrintMessage(ILAsmFilename, ILAsmLineNum,
-								  "duplicate definition for `%s'", name);
-				ILAsmErrors = 1;
-			}
-			else
-			{
-				/* Convert the MemberRef into a FieldDef */
-				ILMemberSetSignature((ILMember *)field, sig);
-				ILMemberSetAttrs((ILMember *)field,
-								 ~((ILUInt32)0), attributes);
-				if((ILField_Token(field) & IL_META_TOKEN_MASK)
-						== IL_META_TOKEN_MEMBER_REF)
-				{
-					ILFieldNewToken(field);
-				}
-			}
-			return field;
+			/* This is a duplicate */
+			ILAsmPrintMessage(ILAsmFilename, ILAsmLineNum,
+							  "duplicate definition for `%s'", name);
+			ILAsmErrors = 1;
 		}
+		else
+		{
+			/* Convert the MemberRef into a FieldDef */
+			ILMemberSetSignature((ILMember *)field, sig);
+			ILMemberSetAttrs((ILMember *)field,
+							 ~((ILUInt32)0), attributes);
+			if((ILField_Token(field) & IL_META_TOKEN_MASK)
+					== IL_META_TOKEN_MEMBER_REF)
+			{
+				ILFieldNewToken(field);
+			}
+		}
+		return field;
 	}
 
 	/* Create a new field block */
@@ -909,7 +947,8 @@ ILField *ILAsmFieldCreate(ILClass *classInfo, const char *name,
 	return field;
 }
 
-void ILAsmAttributeCreate(ILProgramItem *type, ILIntString *value)
+void ILAsmAttributeCreateFor(ILToken ownerToken, ILProgramItem *type,
+							 ILIntString *value)
 {
 	ILAttribute *attr;
 	ILProgramItem *owner;
@@ -971,7 +1010,7 @@ void ILAsmAttributeCreate(ILProgramItem *type, ILIntString *value)
 	}
 
 	/* Attach the attribute to its owner */
-	owner = ILProgramItem_FromToken(ILAsmImage, ILAsmLastToken);
+	owner = ILProgramItem_FromToken(ILAsmImage, ownerToken);
 	if(!owner)
 	{
 		ILAsmPrintMessage(ILAsmFilename, ILAsmLineNum,
@@ -984,6 +1023,38 @@ void ILAsmAttributeCreate(ILProgramItem *type, ILIntString *value)
 	}
 }
 
+void ILAsmAttributeCreate(ILProgramItem *type, ILIntString *value)
+{
+	ILAsmAttributeCreateFor(ILAsmLastToken, type, value);
+}
+
+void ILAsmSecurityCreate(ILInt64 action, const void *str, int len)
+{
+	ILProgramItem *owner;
+	ILDeclSecurity *decl;
+
+	/* Get the owner for the security blob */
+	owner = ILProgramItem_FromToken(ILAsmImage, ILAsmLastToken);
+	if(!owner)
+	{
+		ILAsmPrintMessage(ILAsmFilename, ILAsmLineNum,
+						  "unknown owner for security declaration");
+		ILAsmErrors = 1;
+		return;
+	}
+
+	/* Create the security blob */
+	decl = ILDeclSecurityCreate(ILAsmImage, 0, owner, (ILUInt32)action);
+	if(!decl)
+	{
+		ILAsmOutOfMemory();
+	}
+	if(!ILDeclSecuritySetBlob(decl, str, len))
+	{
+		ILAsmOutOfMemory();
+	}
+}
+
 ILParameter *ILAsmFindParameter(ILMethod *method, ILUInt32 paramNum)
 {
 	ILParameter *param = 0;
@@ -1013,7 +1084,7 @@ void ILAsmAddSemantics(int type, ILToken token)
 	}
 }
 
-void ILAsmDebugLine(ILUInt32 line, char *filename)
+void ILAsmDebugLine(ILUInt32 line, ILUInt32 column, char *filename)
 {
 	if(ILAsmDebugMode)
 	{
@@ -1021,7 +1092,7 @@ void ILAsmDebugLine(ILUInt32 line, char *filename)
 				IL_META_TOKEN_MASK) == IL_META_TOKEN_METHOD_DEF)
 		{
 			/* Debug line information within the body of a method */
-			ILAsmOutDebugLine(filename, line);
+			ILAsmOutDebugLine(filename, line, column);
 		}
 		else
 		{
@@ -1032,7 +1103,7 @@ void ILAsmDebugLine(ILUInt32 line, char *filename)
 			int len = ILMetaCompressData
 				(buf, ILWriterDebugString(ILAsmWriter, filename));
 			len += ILMetaCompressData(buf + len, line);
-			len += ILMetaCompressData(buf + len, 0);		/* column number */
+			len += ILMetaCompressData(buf + len, column);
 			if(item)
 			{
 				ILWriterDebugAdd(ILAsmWriter, item,
