@@ -25,6 +25,8 @@
 extern	"C" {
 #endif
 
+#ifdef IL_CONFIG_JAVA
+
 /* "image/jopcodes.c" */
 extern ILOpcodeInfo const ILJavaOpcodeTable[256];
 
@@ -857,6 +859,8 @@ void ILDAsmDumpJavaMethod(ILImage *image, FILE *outstream,
 	ILMethodFreeExceptions(clauses);
 }
 
+#endif /* IL_CONFIG_JAVA */
+
 #ifdef	__cplusplus
 };
 #endif
