@@ -664,6 +664,27 @@ static void dumpMethodPrototype(FILE *outfile, const char *prefix,
 	fputs(");\n", outfile);
 }
 
+/*
+ * Dump a class in "lookup" form (recursively handle nesting)
+ */
+static void dumpLookupClass(FILE *outfile, ILClass *classInfo)
+{
+	const char *namespace;
+	ILClass *nestedParent=ILClassGetNestedParent(classInfo);
+	if(nestedParent)
+	{
+		dumpLookupClass(outfile,nestedParent);
+		putc('/', outfile);
+	}
+	namespace = ILClass_Namespace(classInfo);
+	if(namespace)
+	{
+		fputs(namespace, outfile);
+		putc('.', outfile);
+	}
+	fputs(ILClass_Name(classInfo), outfile);
+}
+
 /*
  * Dump a type in "lookup" form.
  */
@@ -671,7 +692,6 @@ static void dumpLookupType(FILE *outfile, ILType *type, int topLevel)
 {
 	unsigned long numParams;
 	unsigned long param;
-	const char *namespace;
 
 	if(ILType_IsPrimitive(type))
 	{
@@ -700,25 +720,13 @@ static void dumpLookupType(FILE *outfile, ILType *type, int topLevel)
 	else if(ILType_IsClass(type))
 	{
 		putc('o', outfile);
-		namespace = ILClass_Namespace(ILType_ToClass(type));
-		if(namespace)
-		{
-			fputs(namespace, outfile);
-			putc('.', outfile);
-		}
-		fputs(ILClass_Name(ILType_ToClass(type)), outfile);
+		dumpLookupClass(outfile,ILType_ToClass(type));
 		putc(';', outfile);
 	}
 	else if(ILType_IsValueType(type))
 	{
 		putc('v', outfile);
-		namespace = ILClass_Namespace(ILType_ToClass(type));
-		if(namespace)
-		{
-			fputs(namespace, outfile);
-			putc('.', outfile);
-		}
-		fputs(ILClass_Name(ILType_ToClass(type)), outfile);
+		dumpLookupClass(outfile,ILType_ToClass(type));
 		putc(';', outfile);
 	}
 	else if(type != 0)
@@ -1246,6 +1254,31 @@ typedef struct
 static ClassNameInfo *classTable = 0;
 static unsigned long classTableSize = 0;
 
+/*
+ * Determine if a class is already in the class table.
+ */
+static int classAlreadyPresent(const char *name, const char *nspace)
+{
+	unsigned long posn;
+	for(posn = 0; posn < classTableSize; ++posn)
+	{
+		if(strcmp(name, classTable[posn].name) != 0)
+		{
+			continue;
+		}
+		if(nspace && classTable[posn].namespace &&
+		   !strcmp(nspace, classTable[posn].namespace))
+		{
+			return 1;
+		}
+		else if(nspace == classTable[posn].namespace)
+		{
+			return 1;
+		}
+	}
+	return 0;
+}
+
 /*
  * Load an IL image and print stub information for internalcall methods.
  */
@@ -1275,6 +1308,13 @@ static int processFile(const char *filename, ILContext *context,
 							(image, IL_META_TOKEN_TYPE_DEF | token);
 		if(classInfo)
 		{
+			/* If we already saw this class in another assembly, skip it */
+			if(classAlreadyPresent(ILClass_Name(classInfo),
+								   ILClass_Namespace(classInfo)))
+			{
+				continue;
+			}
+
 			/* Determine if this class has "internalcall" methods */
 			method = 0;
 			while((method = (ILMethod *)ILClassNextMemberByKind
