@@ -72,6 +72,13 @@ int GC_full_freq = 19;	   /* Every 20th collection is a full	*/
 GC_bool GC_need_full_gc = FALSE;
 			   /* Need full GC do to heap growth.	*/
 
+#ifdef THREADS
+  GC_bool GC_world_stopped = FALSE;
+# define IF_THREADS(x) x
+#else
+# define IF_THREADS(x)
+#endif
+
 word GC_used_heap_size_after_full = 0;
 
 char * GC_copyright[] =
@@ -221,10 +228,15 @@ void GC_clear_a_few_frames()
     for (i = 0; i < NWORDS; i++) frames[i] = 0;
 }
 
+/* Heap size at which we need a collection to avoid expanding past	*/
+/* limits used by blacklisting.						*/
+static word GC_collect_at_heapsize = (word)(-1);
+
 /* Have we allocated enough to amortize a collection? */
 GC_bool GC_should_collect()
 {
-    return(GC_adj_words_allocd() >= min_words_allocd());
+    return(GC_adj_words_allocd() >= min_words_allocd()
+	   || GC_heapsize >= GC_collect_at_heapsize);
 }
 
 
@@ -470,6 +482,7 @@ GC_stop_func stop_func;
         GC_cond_register_dynamic_libraries();
 #   endif
     STOP_WORLD();
+    IF_THREADS(GC_world_stopped = TRUE);
 #   ifdef CONDPRINT
       if (GC_print_stats) {
 	GC_printf1("--> Marking for collection %lu ",
@@ -500,6 +513,7 @@ GC_stop_func stop_func;
 		      }
 #		    endif
 		    GC_deficit = i; /* Give the mutator a chance. */
+                    IF_THREADS(GC_world_stopped = FALSE);
 	            START_WORLD();
 	            return(FALSE);
 	    }
@@ -533,6 +547,7 @@ GC_stop_func stop_func;
             (*GC_check_heap)();
         }
     
+    IF_THREADS(GC_world_stopped = FALSE);
     START_WORLD();
 #   ifdef PRINTTIMES
 	GET_TIME(current_time);
@@ -914,22 +929,32 @@ word n;
 #	endif
       }
 #   endif
-    expansion_slop = 8 * WORDS_TO_BYTES(min_words_allocd());
-    if (5 * HBLKSIZE * MAXHINCR > expansion_slop) {
-        expansion_slop = 5 * HBLKSIZE * MAXHINCR;
-    }
+    expansion_slop = WORDS_TO_BYTES(min_words_allocd()) + 4*MAXHINCR*HBLKSIZE;
     if (GC_last_heap_addr == 0 && !((word)space & SIGNB)
         || GC_last_heap_addr != 0 && GC_last_heap_addr < (ptr_t)space) {
         /* Assume the heap is growing up */
         GC_greatest_plausible_heap_addr =
-            GC_max(GC_greatest_plausible_heap_addr,
-                   (ptr_t)space + bytes + expansion_slop);
+            (GC_PTR)GC_max((ptr_t)GC_greatest_plausible_heap_addr,
+                           (ptr_t)space + bytes + expansion_slop);
     } else {
         /* Heap is growing down */
         GC_least_plausible_heap_addr =
-            GC_min(GC_least_plausible_heap_addr,
-                   (ptr_t)space - expansion_slop);
+            (GC_PTR)GC_min((ptr_t)GC_least_plausible_heap_addr,
+                           (ptr_t)space - expansion_slop);
     }
+    /* Force GC before we are likely to allocate past expansion_slop */
+      GC_collect_at_heapsize =
+	  GC_heapsize + expansion_slop - 2*MAXHINCR*HBLKSIZE;
+#   if defined(LARGE_CONFIG)
+      if (GC_collect_at_heapsize < GC_heapsize /* wrapped */)
+	GC_collect_at_heapsize = (word)(-1);
+      if (((ptr_t)GC_greatest_plausible_heap_addr <= (ptr_t)space + bytes
+           || (ptr_t)GC_least_plausible_heap_addr >= (ptr_t)space)
+	  && GC_heapsize > 0) {
+	/* GC_add_to_heap will fix this, but ... */
+	WARN("Too close to address space limit: blacklisting ineffective\n", 0);
+      }
+#   endif
     GC_prev_heap_addr = GC_last_heap_addr;
     GC_last_heap_addr = (ptr_t)space;
     GC_add_to_heap(space, bytes);
@@ -1051,6 +1076,8 @@ int kind;
 	EXIT_GC();
       }
     }
+    /* Successful allocation; reset failure count.	*/
+    GC_fail_count = 0;
     
     return(*flh);
 }
