@@ -195,6 +195,11 @@ ILSysIOHandle ILSysIOOpenFile(const char *path, ILUInt32 mode,
 		/* Not reached */
 	}
 
+#ifdef IL_WIN32_PLATFORM
+	/* Unbelievably, windows defaults to text mode!!! */
+	newAccess |= _O_BINARY;
+#endif
+
 	switch(mode)
 	{
 		case ILFileMode_CreateNew:
