@@ -20,8 +20,11 @@
 
 #include "thr_defs.h"
 
-#ifdef	 IL_USE_WIN32_THREADS
+#ifdef IL_USE_WIN32_THREADS
 
+/* Don't include windows.h here otherwise it will override
+   CreateThread macros from gc.h.  It should be included by thr_choose.h */
+   
 #ifdef	__cplusplus
 extern	"C" {
 #endif
