@@ -92,9 +92,31 @@ int _ILLeaveWait(ILThread *thread, int result)
 	}
 
 	_ILWakeupCancelInterrupt(&(thread->wakeup));
-	thread->state &= ~(IL_TS_WAIT_SLEEP_JOIN | IL_TS_INTERRUPTED);
 	
-	_ILMutexUnlock(&(thread->lock));
+	if ((thread->state & IL_TS_SUSPEND_REQUESTED) != 0)
+	{
+	        thread->state &= ~IL_TS_SUSPEND_REQUESTED;
+		thread->state |= IL_TS_SUSPENDED | IL_TS_SUSPENDED_SELF;
+		thread->resumeRequested = 0;
+
+		thread->state &= ~(IL_TS_INTERRUPTED);
+
+		/* Unlock the thread object prior to suspending */
+		_ILMutexUnlock(&(thread->lock));
+
+		/* Suspend until we receive notification from another thread */
+		_ILThreadSuspendSelf(thread);
+
+		_ILMutexLock(&(thread->lock));		
+		thread->state &= ~(IL_TS_WAIT_SLEEP_JOIN | IL_TS_INTERRUPTED);
+		_ILMutexUnlock(&(thread->lock));
+	}
+	else
+	{	
+		thread->state &= ~(IL_TS_WAIT_SLEEP_JOIN | IL_TS_INTERRUPTED);
+
+		_ILMutexUnlock(&(thread->lock));
+	}
 
 	return result;
 }
