@@ -134,6 +134,177 @@ static int MutexCloseNamed(ILWaitMutexNamed *mutex)
 	return IL_WAITCLOSE_FREE;
 }
 
+/*
+ * Set of prime numbers used for hashtable allocation.
+ */
+static int __PrimeNumbers[] =
+{
+	13, 29, 61, 127, 257, 521
+};
+
+static int __numberOfPrimes = sizeof(__PrimeNumbers) / sizeof(int);
+
+static void AddMutexToWakeup(ILWaitMutex *mutex, _ILWakeup *wakeup)
+{
+	int i, x;
+	ILWaitMutex **ownedMutexes;
+
+	if (wakeup->ownedMutexes == 0)
+	{
+		/* Allocate owned mutexes hashtable */
+		ownedMutexes = wakeup->ownedMutexes = ILCalloc(__PrimeNumbers[0], sizeof(ILWaitMutex *));
+		wakeup->ownedMutexesCapacity = __PrimeNumbers[0];
+	}
+	else
+	{
+		ownedMutexes = wakeup->ownedMutexes;
+	}
+	
+	/* Check if the hashtable is large enough */
+	if (wakeup->ownedMutexesCount >= wakeup->ownedMutexesCapacity)
+	{
+		int newCapacity;
+		ILWaitMutex **newArray;
+
+		/* Grow the new hashtable */
+		
+		if (wakeup->ownedMutexesCapacity >= (IL_MAX_NATIVE_INT) / 2)
+		{
+			return;
+		}
+
+		/* Find a nice efficient prime sized hashtable.  If it gets too large
+		   then forget about using a prime */
+
+		newCapacity = wakeup->ownedMutexesCapacity * 2;
+
+		for (i = __numberOfPrimes - 1; i >= 0; i--)
+		{
+			if (__PrimeNumbers[i] < newCapacity)
+			{
+				if (i == __numberOfPrimes - 1)
+				{
+					break;					
+				}
+				else
+				{
+					newCapacity = __PrimeNumbers[i + 1];
+				}
+			}
+		}
+
+		newArray = ILCalloc(newCapacity, sizeof(ILWaitMutex *));
+
+		if (!newArray)
+		{
+			return;
+		}
+
+		/* Rehash all the existing mutexes into the new table */
+
+		for (i = 0; i < wakeup->ownedMutexesCapacity; i++)
+		{
+			if (ownedMutexes[i] == 0)
+			{
+				/* This bucket doesn't have an entry */
+
+				continue;
+			}
+		
+			/* Find the new available bucket in the new hashtable */
+
+			#if SIZEOF_INT <= 4
+				x = (((int)ownedMutexes[i]) >> 2) % newCapacity;
+			#else
+				x = (((int)ownedMutexes[i]) >> 3) % newCapacity;
+			#endif
+			
+			for (;;)
+			{
+				if (newArray[x] == 0)
+				{
+					newArray[x] = mutex;
+
+					break;
+				}
+
+				x++;
+				x %= newCapacity;
+			}
+			
+		}
+
+		ownedMutexes = wakeup->ownedMutexes = newArray;
+		wakeup->ownedMutexesCapacity = newCapacity;
+	}
+
+	/* Get the initial bucket to try putting the mutex in */
+	
+	#if SIZEOF_INT <= 4
+		i = (((int)mutex) >> 2) % wakeup->ownedMutexesCapacity;
+	#else
+		i = (((int)mutex) >> 3) % wakeup->ownedMutexesCapacity;
+	#endif
+
+	/* Scan the hashtable and find an empty bucket for the mutex */
+
+	for (;;)
+	{
+		if (ownedMutexes[i] == 0)
+		{
+			ownedMutexes[i] = mutex;
+			wakeup->ownedMutexesCount++;
+
+			break;
+		}
+
+		i++;
+		i %= wakeup->ownedMutexesCapacity;
+	}
+}
+
+static void RemoveMutexFromWakeup(ILWaitMutex *mutex, _ILWakeup *wakeup)
+{
+	int i, j;
+	ILWaitMutex **ownedMutexes;
+
+	ownedMutexes = wakeup->ownedMutexes;
+
+	if (ownedMutexes == 0)
+	{
+		return;
+	}
+
+	/* Get the initial (and correct right) bucket */
+
+	#if SIZEOF_INT <= 4
+		j = i = (((int)mutex) >> 2) % wakeup->ownedMutexesCapacity;
+	#else
+		j = i = (((int)mutex) >> 3) % wakeup->ownedMutexesCapacity;
+	#endif
+
+	/* Scan the hashtable and clear the entry for the mutex (if found) */
+
+	for (;;)
+	{
+		if (ownedMutexes[i] == mutex)
+		{
+			ownedMutexes[i] = 0;
+			wakeup->ownedMutexesCount--;
+
+			break;
+		}
+		
+		i++;
+		i %= wakeup->ownedMutexesCapacity;
+		
+		if (i == j)
+		{
+			break;
+		}
+	}
+}
+
 /*
  * Register a wakeup object with a wait mutex.
  */
@@ -150,7 +321,16 @@ static int MutexRegister(ILWaitMutex *mutex, _ILWakeup *wakeup)
 		/* No one owns the mutex, so grab it for ourselves */
 		mutex->owner = wakeup;
 		mutex->count = 1;
+
 		result = IL_WAITREG_ACQUIRED;
+
+		if (mutex->parent.kind == IL_WAIT_MUTEX
+			|| mutex->parent.kind == IL_WAIT_NAMED_MUTEX)
+		{
+			/* Newly aquired mutex so add it to the wakeup's owned mutex list */
+
+			AddMutexToWakeup(mutex, wakeup);
+		}
 	}
 	else if(mutex->owner == wakeup)
 	{
@@ -192,7 +372,27 @@ static void MutexUnregister(ILWaitMutex *mutex, _ILWakeup *wakeup, int release)
 	{
 		if(--(mutex->count) == 0)
 		{
-			mutex->owner = 0;
+			if (mutex->parent.kind == IL_WAIT_MUTEX
+				|| mutex->parent.kind == IL_WAIT_NAMED_MUTEX)
+			{
+				RemoveMutexFromWakeup(mutex, wakeup);
+			}
+
+			mutex->owner = _ILWakeupQueueWake(&(mutex->queue));
+
+			if(mutex->owner != 0)
+			{
+				mutex->count = 1;
+			}
+		}
+	}
+	else if (mutex->owner == wakeup && mutex->count == 1)
+	{
+		if (mutex->parent.kind == IL_WAIT_MUTEX
+			|| mutex->parent.kind == IL_WAIT_NAMED_MUTEX)
+		{
+			/* Newly aquired mutex so add it to the wakeup's owned mutex list */
+			AddMutexToWakeup(mutex, wakeup);		
 		}
 	}
 
@@ -226,7 +426,8 @@ ILWaitHandle *ILWaitMutexCreate(int initiallyOwned)
 	if(initiallyOwned)
 	{
 		mutex->owner = &((ILThreadSelf())->wakeup);
-		mutex->count = 1;
+		mutex->count = 1;		
+		AddMutexToWakeup(mutex, mutex->owner);
 	}
 	else
 	{
@@ -288,6 +489,7 @@ ILWaitHandle *ILWaitMutexNamedCreate(const char *name, int initiallyOwned,
 			if(initiallyOwned)
 			{
 				owned = (ILWaitOne(&(mutex->parent.parent), 0) == 0);
+				AddMutexToWakeup(&(mutex->parent), &(ILThreadSelf()->wakeup));
 			}
 			else
 			{
@@ -331,6 +533,7 @@ ILWaitHandle *ILWaitMutexNamedCreate(const char *name, int initiallyOwned,
 	{
 		mutex->parent.owner = &((ILThreadSelf())->wakeup);
 		mutex->parent.count = 1;
+		AddMutexToWakeup(&(mutex->parent), mutex->parent.owner);
 	}
 	else
 	{
@@ -357,19 +560,22 @@ ILWaitHandle *ILWaitMutexNamedCreate(const char *name, int initiallyOwned,
 
 int ILWaitMutexRelease(ILWaitHandle *handle)
 {
+	_ILWakeup *wakeup;
 	ILWaitMutex *mutex = (ILWaitMutex *)handle;
 	int result;
 
 	/* Lock down the mutex */
 	_ILMutexLock(&(mutex->parent.lock));
 
+	wakeup = &ILThreadSelf()->wakeup;
+
 	/* Determine what to do based on the mutex's state */
 	if((mutex->parent.kind & IL_WAIT_MUTEX) == 0)
 	{
 		/* This isn't actually a mutex */
 		result = IL_WAITMUTEX_RELEASE_FAIL;
 	}
-	else if(mutex->owner != &((ILThreadSelf())->wakeup))
+	else if(mutex->owner != wakeup)
 	{
 		/* This thread doesn't currently own the mutex */
 		result = IL_WAITMUTEX_RELEASE_FAIL;
@@ -379,6 +585,12 @@ int ILWaitMutexRelease(ILWaitHandle *handle)
 		/* The count has returned to zero, so find something
 		   else to give the ownership of the mutex to */
 
+		if (mutex->parent.kind == IL_WAIT_MUTEX
+			|| mutex->parent.kind == IL_WAIT_NAMED_MUTEX)
+		{
+			RemoveMutexFromWakeup(mutex, wakeup);
+		}
+
 		mutex->owner = _ILWakeupQueueWake(&(mutex->queue));
 		if(mutex->owner != 0)
 		{
